from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from DataValue cimport *
from ConvexHull2D cimport *
from PeptideIdentification cimport *
from BaseFeature cimport *

from UniqueIdInterface cimport *

cdef extern from "<OpenMS/KERNEL/Feature.h>" namespace "OpenMS":

    cdef cppclass Feature(UniqueIdInterface, RichPeak2D):
        #
        # wrap-inherits:
        #    UniqueIdInterface
        #    RichPeak2D
        #
        # wrap-doc:
        #   An LC-MS feature.
        #   -----
        #   The Feature class is used to describe the two-dimensional signal caused by an
        #   analyte. It can store a charge state and a list of peptide identifications
        #   (for peptides). The area occupied by the Feature in the LC-MS data set is
        #   represented by a list of convex hulls (one for each isotopic peak). There is
        #   also a convex hull for the entire Feature. The model description can store
        #   the parameters of a two-dimensional theoretical model of the underlying
        #   signal in LC-MS. Currently, non-peptide compounds are also represented as
        #   features.

        Feature() nogil except +
        Feature(Feature &) nogil except +

        float getQuality(Size index) nogil except +
        void setQuality(Size index, float q) nogil except +
        float getOverallQuality() nogil except +
        void setOverallQuality(float q) nogil except +

        libcpp_vector[Feature] getSubordinates() nogil except +
        void setSubordinates(libcpp_vector[Feature]) nogil except +

        bool encloses(double rt, double mz) nogil except +
        ConvexHull2D getConvexHull() nogil except +
        libcpp_vector[ConvexHull2D] getConvexHulls() nogil except +
        void setConvexHulls(libcpp_vector[ConvexHull2D]) nogil except +

        bool operator==(Feature) nogil except +
        bool operator!=(Feature) nogil except +

        # from BaseFeature

        # float getQuality()  nogil except +
        # void setQuality(float q) nogil except +

        float getWidth() nogil except +
        void setWidth(float q) nogil except +

        Int getCharge() nogil except +
        void setCharge(Int q) nogil except +
        AnnotationState getAnnotationState() nogil except +

        # returns a mutable reference to the PeptideIdentification vector
        libcpp_vector[PeptideIdentification] getPeptideIdentifications() nogil except +
        # sets the PeptideIdentification vector
        void setPeptideIdentifications(libcpp_vector[PeptideIdentification] & peptides) nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from PeakShape cimport *
from OptimizePick cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/OptimizePeakDeconvolution.h>" namespace "OpenMS":
    
    cdef cppclass OptimizePeakDeconvolution(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        OptimizePeakDeconvolution() nogil except +
        OptimizePeakDeconvolution(OptimizePeakDeconvolution) nogil except +
        PenaltyFactorsIntensity  getPenalties() nogil except +
        void setPenalties(PenaltyFactorsIntensity & penalties) nogil except +
        Int getCharge() nogil except +
        void setCharge(Int charge) nogil except +
        bool optimize(libcpp_vector[ PeakShape ] & peaks, OptimizePeakDeconvolution_Data & data) nogil except +
        Size getNumberOfPeaks_(Int charge, libcpp_vector[ PeakShape ] & temp_shapes, OptimizePeakDeconvolution_Data & data) nogil except +

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/OptimizePeakDeconvolution.h>" namespace "OpenMS::OptimizationFunctions":
    
    cdef cppclass PenaltyFactorsIntensity(OptimizationFunctions_PenaltyFactors):
        # wrap-inherits:
        #  OptimizationFunctions_PenaltyFactors
        PenaltyFactorsIntensity() nogil except +
        PenaltyFactorsIntensity(PenaltyFactorsIntensity) nogil except +
        double height


cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/OptimizePeakDeconvolution.h>" namespace "OpenMS::OptimizePeakDeconvolution":

    cdef cppclass OptimizePeakDeconvolution_Data "OpenMS::OptimizePeakDeconvolution::Data":
      OptimizePeakDeconvolution_Data() nogil except +
      OptimizePeakDeconvolution_Data(OptimizePeakDeconvolution_Data) nogil except + # no-wrap

      libcpp_vector[PeakShape] peaks
      libcpp_vector[double] positions
      libcpp_vector[double] signal
      PenaltyFactorsIntensity penalties 
      Int charge 




from Types cimport *
from String cimport *

cdef extern from "<OpenMS/METADATA/Gradient.h>" namespace "OpenMS":

    cdef cppclass Gradient:

        Gradient() nogil except +
        Gradient(Gradient) nogil except + # wrap-ignore

        #   @brief Adds an eluent at the end of the eluent array
        #
        #   @exception Exception::InvalidValue is thrown if the same eluent name is used twice.
        void addEluent(String eluent) nogil except +
        # removes all eluents
        void clearEluents() nogil except +
        # returns a reference to the list of eluents
        libcpp_vector[String] getEluents() nogil except +

        #   @brief Adds a timepoint at the end of the timepoint array
        #
        #   @exception Exception::OutOfRange is thrown if the new timpoint is before the last timepoint.
        void addTimepoint(Int timepoint) nogil except +
        # removes all timepoints
        void clearTimepoints() nogil except +
        # returns a reference to the list of timepoints
        libcpp_vector[Int] getTimepoints() nogil except +

        #   @brief sets the percentage of eluent @p eluent at timepoint @p timepoint
        #
        #   @exception Exception::InvalidValue is thrown if the eluent, timepoint or percentage is invalid.
        void setPercentage(String eluent, Int timepoint, UInt percentage) nogil except +
        UInt getPercentage(String eluent, Int timepoint) nogil except +

        # TODO 
        # libcpp_vector[ libcpp_vector[unsigned int] ] getPercentages() nogil except +

        # sets all percentage values to 0
        void clearPercentages() nogil except +
        # checks if the percentages of all timepoints add up to 100%
        bool isValid() nogil except +




from Types cimport *
from libcpp cimport bool
from PeakShape cimport *
# from OptimizePeakDeconvolution cimport *
from MSExperiment cimport *
from MSSpectrum cimport *
# from PeakIndex cimport *
from IsotopeCluster cimport *
from DefaultParamHandler cimport *
# from OptimizePick cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/TwoDOptimization.h>" namespace "OpenMS":
    
    cdef cppclass TwoDOptimization(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        TwoDOptimization() nogil except +
        TwoDOptimization(TwoDOptimization) nogil except +

        double getMZTolerance() nogil except +
        void setMZTolerance(double tolerance_mz) nogil except +
        double getMaxPeakDistance() nogil except +
        void setMaxPeakDistance(double max_peak_distance) nogil except +
        UInt getMaxIterations() nogil except +
        void setMaxIterations(UInt max_iteration) nogil except +
        # NAMESPACE # OptimizationFunctions::PenaltyFactorsIntensity  getPenalties() nogil except +
        # NAMESPACE # void setPenalties(OptimizationFunctions::PenaltyFactorsIntensity & penalties) nogil except +
        # TEMPLATE # void optimize(InputSpectrumIterator first, InputSpectrumIterator last, MSExperiment & ms_exp, bool real2D) nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from IdentificationData cimport *
from FASTAFile cimport *
from MzTab cimport *
from PeptideIdentification cimport *
from ProteinIdentification cimport *

cdef extern from "<OpenMS/METADATA/ID/IdentificationDataConverter.h>" namespace "OpenMS":
    
    cdef cppclass IdentificationDataConverter "OpenMS::IdentificationDataConverter":
        IdentificationDataConverter() nogil except +
        IdentificationDataConverter(IdentificationDataConverter) nogil except + #wrap-ignore
        void importIDs(IdentificationData & id_data, const libcpp_vector[ ProteinIdentification ] & proteins, const libcpp_vector[ PeptideIdentification ] & peptides) nogil except +
        void exportIDs(const IdentificationData & id_data, libcpp_vector[ ProteinIdentification ] & proteins, libcpp_vector[ PeptideIdentification ] & peptides, bool export_oligonucleotides) nogil except +
        MzTab exportMzTab(const IdentificationData & id_data) nogil except +
        #void importSequences(IdentificationData & id_data, const libcpp_vector[ FASTAFile::FASTAEntry ] & fasta, IdentificationData::MoleculeType type_, const String & decoy_pattern) nogil except +




from Types cimport *
from TargetedExperiment cimport *
from ProgressLogger cimport *
from MRMIonSeries cimport *
from ModificationsDB cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MRMAssay.h>" namespace "OpenMS":
    
    cdef cppclass MRMAssay(ProgressLogger) :
        # wrap-inherits:
        #  ProgressLogger

        MRMAssay() nogil except +
        MRMAssay(MRMAssay) nogil except + #wrap-ignore

        void reannotateTransitions(TargetedExperiment & exp, 
                                   double precursor_mz_threshold,
                                   double product_mz_threshold,
                                   libcpp_vector[ String ] fragment_types, 
                                   libcpp_vector[ size_t ] fragment_charges, 
                                   bool enable_specific_losses, 
                                   bool enable_unspecific_losses,
                                   int round_decPow) nogil except +

        void restrictTransitions(TargetedExperiment & exp, 
                                 double lower_mz_limit,
                                 double upper_mz_limit,
                                 libcpp_vector[ libcpp_pair[ double, double ] ] swathes) nogil except +

        void detectingTransitions(TargetedExperiment & exp,
                                  int min_transitions,
                                  int max_transitions) nogil except +

        void detectingTransitionsCompound(TargetedExperiment & exp,
                                          int min_transitions,
                                          int max_transitions) nogil except +
        
        void uisTransitions(TargetedExperiment & exp, 
                            libcpp_vector[ String ] fragment_types,
                            libcpp_vector[ size_t ] fragment_charges,
                            bool enable_specific_losses,
                            bool enable_unspecific_losses,
                            bool enable_ms2_precursors,
                            double mz_threshold,
                            libcpp_vector[ libcpp_pair[ double, double ] ] swathes,
                            int round_decPow,
                            size_t max_num_alternative_localizations,
                            int shuffle_seed) nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Feature cimport *
from DefaultParamHandler cimport *
from TransformationModelLowess cimport *
from MSExperiment cimport *
from ConsensusMap cimport *
from FeatureMap cimport *
from BaseFeature cimport *
# from KDTree cimport *
# from KDTreeFeatureNode cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/KDTreeFeatureMaps.h>" namespace "OpenMS":

    cdef cppclass KDTreeFeatureMaps(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        KDTreeFeatureMaps() nogil except +
        KDTreeFeatureMaps(KDTreeFeatureMaps) nogil except + #wrap-ignore
        KDTreeFeatureMaps(libcpp_vector[ FeatureMap ] & maps, Param & param) nogil except +
        KDTreeFeatureMaps(libcpp_vector[ ConsensusMap ] & maps, Param & param) nogil except +
        void addMaps(libcpp_vector[ FeatureMap ] & maps) nogil except +
        void addMaps(libcpp_vector[ ConsensusMap ] & maps) nogil except +
        # POINTER # void addFeature(Size mt_map_index, BaseFeature * feature) nogil except +
        # POINTER # BaseFeature * feature(Size i) nogil except +
        double rt(Size i) nogil except +
        double mz(Size i) nogil except +
        float intensity(Size i) nogil except +
        Int charge(Size i) nogil except +
        Size mapIndex(Size i) nogil except +
        Size size() nogil except +
        Size treeSize() nogil except +
        Size numMaps() nogil except +
        void clear() nogil except +
        void optimizeTree() nogil except +
        void getNeighborhood(Size index,
                             libcpp_vector[ size_t ] & result_indices,
                             double rt_tol,
                             double mz_tol,
                             bool mz_ppm,
                             bool include_features_from_same_map,
                             double max_pairwise_log_fc) nogil except +
        void queryRegion(double rt_low, double rt_high, double mz_low, double mz_high, libcpp_vector[ size_t ] & result_indices, Size ignored_map_index) nogil except +
        # void applyTransformations(libcpp_vector[ TransformationModelLowess * ] & trafos) nogil except +




from Feature cimport *
from FeatureMap cimport *
from ConsensusMap cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/IDConflictResolverAlgorithm.h>" namespace "OpenMS":

    cdef cppclass IDConflictResolverAlgorithm(DefaultParamHandler):
        IDConflictResolverAlgorithm() nogil except +
        
        IDConflictResolverAlgorithm(IDConflictResolverAlgorithm) nogil except + # wrap-ignore

        void resolve(FeatureMap& features) nogil except +

        void resolve(ConsensusMap& features) nogil except +

        void resolveBetweenFeatures(FeatureMap& features) nogil except +

        void resolveBetweenFeatures(ConsensusMap& features) nogil except +



from AASequence cimport *

cdef extern from "<OpenMS/CHEMISTRY/AAIndex.h>" namespace "OpenMS":
    
    cdef cppclass AAIndex "OpenMS::AAIndex":
        AAIndex(AAIndex) nogil except + #wrap-ignore
        double aliphatic(char aa) nogil except +
        double acidic(char aa) nogil except +
        double basic(char aa) nogil except +
        double polar(char aa) nogil except +
        double getKHAG800101(char aa) nogil except +
        double getVASM830103(char aa) nogil except +
        double getNADH010106(char aa) nogil except +
        double getNADH010107(char aa) nogil except +
        double getWILM950102(char aa) nogil except +
        double getROBB760107(char aa) nogil except +
        double getOOBM850104(char aa) nogil except +
        double getFAUJ880111(char aa) nogil except +
        double getFINA770101(char aa) nogil except +
        double getARGP820102(char aa) nogil except +
        double calculateGB(AASequence &seq, double T) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from String cimport *
from Peak1D cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/FORMAT/MSPFile.h>" namespace "OpenMS":

    cdef cppclass MSPFile:

        MSPFile() nogil except +

        void store(String filename, MSExperiment & exp) nogil except +
        void load(String filename, libcpp_vector[PeptideIdentification] & ids, MSExperiment & exp) nogil except +



from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool
from CVReference cimport *
from CVMappingRule cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/CVMappings.h>" namespace "OpenMS":

    cdef cppclass CVMappings:

        CVMappings() nogil except +
        CVMappings(CVMappings) nogil except +
        void setMappingRules(libcpp_vector[ CVMappingRule ]& cv_mapping_rules) nogil except +
        libcpp_vector[ CVMappingRule ]  getMappingRules() nogil except +
        void addMappingRule(CVMappingRule& cv_mapping_rule) nogil except +
        void setCVReferences(libcpp_vector[ CVReference ]& cv_references) nogil except +
        libcpp_vector[ CVReference ] getCVReferences() nogil except +
        void addCVReference(CVReference& cv_reference) nogil except +
        bool hasCVReference(const String& identifier) nogil except +




from FeatureGroupingAlgorithm cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Feature cimport *
from FeatureMap cimport *
from ConsensusMap cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/FeatureGroupingAlgorithmQT.h>" namespace "OpenMS":

    cdef cppclass FeatureGroupingAlgorithmQT(FeatureGroupingAlgorithm):
        # wrap-inherits:
        #    FeatureGroupingAlgorithm

        FeatureGroupingAlgorithmQT() nogil except +

        # not in FeatureGroupingAlgorithm, as cython has a problem with
        # overloaded methods in base classes
        void group(libcpp_vector[FeatureMap] & maps,
                   ConsensusMap & out
                  ) nogil except +

        void group(libcpp_vector[ConsensusMap] & maps,
                   ConsensusMap & out
                  ) nogil except +

        # Creates a new instance of this class (for Factory)
        # FeatureGroupingAlgorithm * create() nogil except +

        # Returns the product name (for the Factory)
        String getProductName() nogil except +





    def dia_by_ion_score(self, OSSpectrum spectrum , AASequence sequence ,  charge , float bseries_score , float yseries_score ):
        assert isinstance(spectrum, OSSpectrum), 'arg spectrum wrong type'
        assert isinstance(sequence, AASequence), 'arg sequence wrong type'
        assert isinstance(charge, (int, long)), 'arg charge wrong type'
        assert isinstance(bseries_score, float), 'arg bseries_score wrong type'
        assert isinstance(yseries_score, float), 'arg yseries_score wrong type'
        cdef shared_ptr[_OSSpectrum] input_spectrum = spectrum.inst

        cdef double input_bseries_score = (<double>bseries_score)
        cdef double input_yseries_score = (<double>yseries_score)
        self.inst.get().dia_by_ion_score(input_spectrum, (deref(sequence.inst.get())), (<int>charge), input_bseries_score, input_yseries_score)
        yseries_score = input_yseries_score
        bseries_score = input_bseries_score
        return (bseries_score,yseries_score)





from Types cimport *
from DefaultParamHandler cimport *
from FeatureMap cimport *
from FeatureFinderAlgorithmPickedHelperStructs cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/ElutionModelFitter.h>" namespace "OpenMS":
    
    cdef cppclass ElutionModelFitter(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        ElutionModelFitter() nogil except +
        ElutionModelFitter(ElutionModelFitter) nogil except + #wrap-ignore
        void fitElutionModels(FeatureMap & features) nogil except +




from MSExperiment  cimport *
from MSSpectrum  cimport *
from Peak1D  cimport *
from ChromatogramPeak  cimport *
from MSChromatogram  cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from String cimport *
from ProgressLogger cimport *
from PeakFileOptions cimport *
from IMSDataConsumer cimport *

cdef extern from "<OpenMS/FORMAT/HANDLERS/MzMLSqliteHandler.h>" namespace "OpenMS::Internal":

    cdef cppclass MzMLSqliteHandler:

        # MzMLSqliteHandler() nogil except +

        MzMLSqliteHandler(String filename) nogil except +
        MzMLSqliteHandler(MzMLSqliteHandler h) nogil except +

        void readExperiment(MSExperiment & exp, bool meta_only )  nogil except +
  
        void readSpectra(libcpp_vector[MSSpectrum] & exp, libcpp_vector[int] indices, bool meta_only ) nogil except +

        void readChromatograms(libcpp_vector[MSChromatogram] & exp, libcpp_vector[int] indices, bool meta_only ) nogil except +
  
        Size getNrSpectra() nogil except +
  
        Size getNrChromatograms() nogil except +
  
        void setConfig(bool write_full_meta, bool use_lossy_compression, double linear_abs_mass_acc)  nogil except +
  
        libcpp_vector[size_t] getSpectraIndicesbyRT(double RT, double deltaRT, libcpp_vector[int] indices) nogil except +
  
        void writeExperiment(MSExperiment exp) nogil except +
  
        void createTables() nogil except +
  
        void writeSpectra(libcpp_vector[MSSpectrum] spectra) nogil except +
  
        void writeChromatograms(libcpp_vector[MSChromatogram] chroms) nogil except +
  
        void writeRunLevelInformation(MSExperiment exp, bool write_full_meta, int run_id) nogil except +




from String cimport String as _String
cdef shared_ptr[_String] convString(argument_var)
cdef convOutputString(_String s)



cdef inline convOutputString(_String _r):
    # Generic function to convert OpenMS::String to Python unicode strings
    #
    # This assumes that all data encoded in OpenMS::String is encoded using
    # UTF8 (in practice OpenMS does not make any assumption about encoding and
    # OpenMS::String simply contains a byte sequence, that is implicitely
    # assumed to be ASCII encoded).
    #
    cdef char* c_string = _cast_const_away(<char*> _r.c_str())
    cdef Py_ssize_t length = _r.length()

    try:
        py_result = c_string[:length].decode('UTF-8')
    except UnicodeDecodeError:
        py_result = c_string[:length]

    return py_result

cdef inline shared_ptr[_String] convString(argument_var):
    # Generic function to convert Python strings to OpenMS::String
    # 
    # This allows us to either extract the already existing shared_ptr from a
    # Python-type OpenMS::String holder or create a new shared_ptr from a
    # Python string.
    # In case the user only provides a str, unicode or bytes argument, we will
    # have to create a new OpenMS::String object, the shared_ptr will
    # automatically delete it after it has been used for a function call
    # (presumably the user does not want to have the value returned by
    # reference).
    #
    # Note: even though Python 3.x does not know the unicode keyword,
    # Cython does and uses the PyUnicode_Check call. 
    #
    cdef char* c_string_argument_var
    cdef shared_ptr[_String] res
    if isinstance(argument_var, String):
        res = (<String>argument_var).inst
        return res
    elif isinstance(argument_var, bytes):
        # Simple, convert directly to char* - however there may be zero bytes
        # in the array when using certain encodings, we therefore need to make
        # sure we capture the full length.
        res = shared_ptr[_String](new _String(<char*>argument_var, len(argument_var)))
        return res
    elif isinstance(argument_var, str) or isinstance(argument_var, unicode):
        # First encode with UTF8 (note that output encoding as well as
        # String.toString both decode with UTF8), then convert the result to
        # char*
        py_byte_string = argument_var.encode('UTF-8')
        c_string_argument_var = py_byte_string
        res = shared_ptr[_String](new _String(<char*>c_string_argument_var))
        return res
    else:
        raise Exception("Can only convert the following types to String: pyopenms.String, bytes, str, unicode")




from Types cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from ISpectrumAccess cimport *
# from ITransition cimport *
# from TransitionExperiment cimport *
# from StatsHelpers cimport *
# from Scoring cimport *
from LightTargetedExperiment cimport *

cdef extern from "<OpenMS/OPENSWATHALGO/ALGO/MRMScoring.h>" namespace "OpenSwath":
    
    cdef cppclass MRMScoring "OpenSwath::MRMScoring":
        MRMScoring(MRMScoring) nogil except + #wrap-ignore

        # TODO create class for XCorrMatrix
        # XCorrMatrixType  getXCorrMatrix() nogil except +
        # NAMESPACE # # POINTER # void initializeXCorrMatrix(OpenSwath::IMRMFeature * mrmfeature, OpenSwath::ITransitionGroup * transition_group, bool normalize) nogil except +
        double calcXcorrCoelutionScore() nogil except +
        double calcXcorrCoelutionWeightedScore(libcpp_vector[ double ] & normalized_library_intensity) nogil except +
        libcpp_vector[ double ] calcSeparateXcorrContrastCoelutionScore() nogil except +
        double calcXcorrPrecursorContrastCoelutionScore() nogil except +

        double calcXcorrShapeScore() nogil except +
        double calcXcorrShapeWeightedScore(libcpp_vector[ double ] & normalized_library_intensity) nogil except +
        libcpp_vector[ double ] calcSeparateXcorrContrastShapeScore() nogil except +
        double calcXcorrPrecursorContrastShapeScore() nogil except +

        # NAMESPACE # # POINTER # void calcLibraryScore(OpenSwath::IMRMFeature * mrmfeature, libcpp_vector[ TransitionType ] & transitions, double & correlation, double & rmsd, double & manhattan, double & dotprod) nogil except +
        double calcRTScore(LightCompound & peptide, double normalized_experimental_rt) nogil except +
        # NAMESPACE # # POINTER # double calcSNScore(OpenSwath::IMRMFeature * mrmfeature, libcpp_vector[ OpenSwath::ISignalToNoisePtr ] & signal_noise_estimators) nogil except +

        double calcMIScore() nogil except +
        double calcMIWeightedScore(const libcpp_vector[ double ] & normalized_library_intensity) nogil except +
        
        double calcMIPrecursorScore() nogil except + 
        double calcMIPrecursorContrastScore() nogil except + 
        double calcMIPrecursorCombinedScore() nogil except + 
        libcpp_vector[ double ] calcSeparateMIContrastScore() nogil except +              
        
        libcpp_vector[ libcpp_vector[ double ] ]  getMIMatrix() nogil except +




# @brief Mathematical and physical constants namespace.
# 
# This namespace contains definitions for some basic mathematical and physical constants.
# All constants are double precision.

# EPSILON (used for comparisons)
EPSILON = 1e-6

# PI
PI = 3.14159265358979323846

# Euler's number - base of the natural logarithm
E = 2.718281828459045235

# Elementary charge
ELEMENTARY_CHARGE = 1.60217738E-19 # C

# Elementary charge (alias)
e0 = ELEMENTARY_CHARGE

# Electron mass
ELECTRON_MASS = 9.1093897E-31 # kg

# Electron mass in units
ELECTRON_MASS_U = 1.0 / 1822.8885020477 # u

# Proton mass
PROTON_MASS = 1.6726230E-27 # kg

# Proton mass in units
PROTON_MASS_U = 1.0072764667710 # u

# Mass difference between Carbon-13 and Carbon-12 in units
C13C12_MASSDIFF_U = 1.0033548378 # u

# Neutron mass
NEUTRON_MASS = 1.6749286E-27 # kg

# Neutron mass in units
NEUTRON_MASS_U = 1.00866491566 # u

# Avogadro constant
AVOGADRO = 6.0221367E+23 # 1 / mol

# Avogadro constant (alias)
NA = AVOGADRO

# Avogadro constant (alias)
MOL = AVOGADRO

# Boltzmann constant
BOLTZMANN = 1.380657E-23 # J / K

# Boltzmann constant (alias)
k = BOLTZMANN

# Planck constant
PLANCK = 6.6260754E-34 # J * sec

# Planck constant (alias)
h = PLANCK

# Gas constant (= NA * k)
GAS_CONSTANT = NA * k

# Gas constant (alias)
R = GAS_CONSTANT

# Faraday constant (= NA * e0)
FARADAY = NA * e0

# Faraday constant (alias)
F = FARADAY

# Bohr radius
BOHR_RADIUS = 5.29177249E-11 # m

# Bohr radius (alias)
a0 = BOHR_RADIUS

#  the following values from:
#  P.W.Atkins: Physical Chemistry, 5th ed., Oxford University Press, 1995

# Vacuum permittivity
VACUUM_PERMITTIVITY = 8.85419E-12 # C^2 / (J * m)

# Vacuum permeability
VACUUM_PERMEABILITY = (4 * PI * 1E-7) # J s^2 / (C^2 * m)

# Speed of light
SPEED_OF_LIGHT = 2.99792458E+8 # m / s

# Speed of Light (alias)
c = SPEED_OF_LIGHT

# Gravitational constant
GRAVITATIONAL_CONSTANT  = 6.67259E-11 # N m^2 / kg^2

# Fine structure constant
FINE_STRUCTURE_CONSTANT = 7.29735E-3 # 1

# Degree per rad
DEG_PER_RAD = 57.2957795130823209

# Rad per degree
RAD_PER_DEG = 0.0174532925199432957

# mm per inch
MM_PER_INCH = 25.4

# m per foot
M_PER_FOOT = 3.048

# Joule per calorie
JOULE_PER_CAL  = 4.184

# Calories per Joule
CAL_PER_JOULE = (1 / 4.184)

# User parameter name for precursor mz error in ppm
PRECURSOR_ERROR_PPM_USERPARAM = "precursor_mz_error_ppm"

# User parameter name for precursor mz error in ppm
FRAGMENT_ANNOTATION_USERPARAM = "fragment_annotation"




import os
import glob
import pyopenms

toTest = set()

# pyopenms in the ignore list as it is represented twice in
# the pyopenms package
ignore = ["numpy", "np", "re", "os", "types", "sysinfo", "pyopenms"]

for clz_name, clz in pyopenms.__dict__.items():
    if clz_name in ignore or clz_name.startswith("__"):
        continue

    if not hasattr(clz, "__dict__"):
        continue
    for method_name, method in clz.__dict__.items():

        if method_name.startswith("_") and not method_name.startswith("__"):
            continue

        if method_name in [ "__doc__", "__new__", "__file__", "__name__",
                "__package__", "__builtins__", "__copy__"]:
            continue

        toTest.add("%s.%s" % (clz_name, method_name))

def parse_doc(item, collection):
    if item.__doc__ is not None:
       it = iter(item.__doc__.split("\n"))
       for line in it:
            if not "@tests" in line:
                continue
            for line in it:
                line = line.strip()
                if "@end" in line:
                    break
                if not line:
                    continue
                clz, method = line.split(".")
                if clz=="":
                    clz = oldclzz
                fullname = "%s.%s" % (clz, method)
                if fullname.endswith("()"):
                    print fullname, "declared with parentesis, fix it"
                    fullname = fullname[:-2]
                collection.add(fullname)
                oldclzz = clz

def collectRecursed(obj, collection):
    if hasattr(obj, "__dict__"):
        for name, item in obj.__dict__.items():
                if name.upper().startswith("TEST") or\
                    name.upper().startswith("_TEST"):
                    parse_doc(item, collection)
                    collectRecursed(item, collection)

declaredAsTested = set()

for p in glob.glob("tests/unittests/test*.py"):
    module_name= p[:-3].replace("/",".").replace(os.sep, ".")

    module = __import__(module_name).unittests
    collectRecursed(module, declaredAsTested)

missing = toTest-declaredAsTested

if missing:
    print
    print len(missing), "tests/test declarations  are missing !"
    for name in sorted(missing):
        print "    ", name

toMuch = declaredAsTested-toTest

if toMuch:
    print
    print len(toMuch), "tests/test declarations do not fit:"
    for name in toMuch:
        print "    ", name



from Types cimport *
from Param cimport *
from String cimport *
from libcpp.pair cimport pair as libcpp_pair
from libcpp.vector cimport vector as libcpp_vector
from libcpp.string cimport string as String

from TransformationModel cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/TransformationModelLinear.h>" namespace "OpenMS":


    cdef cppclass TransformationModelLinear(TransformationModel):
        # wrap-inherits:
        #   TransformationModel

        TransformationModelLinear(TransformationModelLinear) nogil except + #wrap-ignore
        TransformationModelLinear(libcpp_vector[TM_DataPoint]& data, Param& params) nogil except +

        void getDefaultParameters(Param&)

        double evaluate(double value) nogil except +
        # void getParameters(double & slope, double & intercept, String& x_weight, String& y_weight, double & x_datum_min, double & x_datum_max, double & y_datum_min, double & y_datum_max) nogil except +
        void invert() nogil except +




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from SimTypes cimport *
from SVMWrapper cimport *
from AASequence cimport *

cdef extern from "<OpenMS/CHEMISTRY/SvmTheoreticalSpectrumGenerator.h>" namespace "OpenMS":
    
    cdef cppclass SvmTheoreticalSpectrumGenerator "OpenMS::SvmTheoreticalSpectrumGenerator":
        SvmTheoreticalSpectrumGenerator() nogil except +
        SvmTheoreticalSpectrumGenerator(SvmTheoreticalSpectrumGenerator) nogil except +
        # void simulate(MSSpectrum &spectrum, AASequence &peptide, boost::random::mt19937_64& rng, Size precursor_charge) nogil except +
        void load() nogil except +
        libcpp_vector[ IonType ]  getIonTypes() nogil except +

cdef extern from "<OpenMS/CHEMISTRY/SvmTheoreticalSpectrumGenerator.h>" namespace "OpenMS::SvmTheoreticalSpectrumGenerator":
    
    cdef cppclass SvmModelParameterSet "OpenMS::SvmTheoreticalSpectrumGenerator::SvmModelParameterSet":

        SvmModelParameterSet(SvmModelParameterSet) nogil except + #wrap-ignore

        # libcpp_vector[ shared_ptr[ SVMWrapper ] ] class_models
        # libcpp_vector[ shared_ptr[ SVMWrapper ] ] reg_models
        # TODO STL map with wrapped key
        # libcpp_map[ ResidueType, double ] _intensities
        # libcpp_vector[ IonType ] ion_types
        # TODO STL map with wrapped key
        # libcpp_map[ IonType, libcpp_vector[ IonType ] ] secondary_types
        Size number_intensity_levels
        Size number_regions
        libcpp_vector[ double ] feature_max
        libcpp_vector[ double ] feature_min
        double scaling_lower
        double scaling_upper
        libcpp_vector[ double ] intensity_bin_boarders
        libcpp_vector[ double ] intensity_bin_values
        # TODO nested STL
        # libcpp_map[ libcpp_pair[ IonType, Size ], libcpp_vector[ libcpp_vector[ double ] ] ] conditional_prob

cdef extern from "<OpenMS/CHEMISTRY/SvmTheoreticalSpectrumGenerator.h>" namespace "OpenMS::SvmTheoreticalSpectrumGenerator":
    
    cdef cppclass IonType "OpenMS::SvmTheoreticalSpectrumGenerator::IonType":
        IonType() nogil except +
        IonType(IonType) nogil except +
        IonType(ResidueType residue, EmpiricalFormula l, Int charge) nogil except +
        bool operator<(IonType &rhs) nogil except +
        ResidueType residue
        EmpiricalFormula loss
        Int charge

# cdef extern from "<OpenMS/CHEMISTRY/SvmTheoreticalSpectrumGenerator.h>" namespace "OpenMS::SvmTheoreticalSpectrumGenerator":
#     
#     cdef cppclass DescriptorSet "OpenMS::SvmTheoreticalSpectrumGenerator::DescriptorSet":
#         DescriptorSet(DescriptorSet) nogil except + #wrap-ignore
#         libcpp_vector[svm_node] descriptors # we would have to wrap libsvm for this
# 
# 



from smart_ptr cimport shared_ptr
from TargetedExperiment cimport *
from TargetedExperimentHelper cimport *
from OpenSwathDataStructures cimport *
from LightTargetedExperiment cimport LightTargetedExperiment, LightCompound

from MSSpectrum cimport *
from Peak1D cimport *
from MSChromatogram cimport *
from ChromatogramPeak cimport *
from AASequence cimport *

# this class has addons, see the ./addons folder (../addons/OpenSwathDataAccessHelper.pyx)

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/DATAACCESS/DataAccessHelper.h>" namespace "OpenMS":

    cdef cppclass OpenSwathDataAccessHelper:

        # OpenSwathDataAccessHelper() nogil except +
        # OpenSwathDataAccessHelper(OpenSwathDataAccessHelper) nogil except + # wrap-ignore

        # Convert a SpectrumPtr to an OpenMS Spectrum
        void convertToOpenMSSpectrum(shared_ptr[OSSpectrum] sptr, MSSpectrum & spectrum) nogil except +

        # Convert an OpenMS Spectrum to an SpectrumPtr
        shared_ptr[OSSpectrum] convertToSpectrumPtr(MSSpectrum spectrum) nogil except + # wrap-ignore

        # Convert an OpenMS Chromatogram to an ChromatogramPtr
        shared_ptr[OSChromatogram] convertToChromatogramPtr(MSChromatogram chromatogram) nogil except + # wrap-ignore

        # Convert a ChromatogramPtr to an OpenMS Chromatogram
        void convertToOpenMSChromatogram(shared_ptr[OSChromatogram] cptr, MSChromatogram & chromatogram) nogil except +
        void convertToOpenMSChromatogramFilter(MSChromatogram & chromatogram, shared_ptr[OSChromatogram] cptr,
                                               double rt_min, double rt_max) nogil except +

        # convert from the OpenMS Targeted experiment to the light Targeted Experiment
        void convertTargetedExp(TargetedExperiment & transition_exp_, LightTargetedExperiment & transition_exp) nogil except +

        # convert from the LightCompound to an OpenMS AASequence (with correct modifications)
        void convertPeptideToAASequence(LightCompound & peptide, AASequence & aa_sequence) nogil except +
        
        # convert from the OpenMS TargetedExperiment Peptide to the LightTargetedExperiment Peptide
        void convertTargetedCompound(Peptide pep, LightCompound & p) nogil except +




from Types cimport *
from AbsoluteQuantitationMethod cimport *

cdef extern from "<OpenMS/FORMAT/AbsoluteQuantitationMethodFile.h>" namespace "OpenMS":

    cdef cppclass AbsoluteQuantitationMethodFile:

        AbsoluteQuantitationMethodFile()  nogil except +
        AbsoluteQuantitationMethodFile(AbsoluteQuantitationMethodFile)  nogil except + #wrap-ignore

        void load(const String& filename, libcpp_vector[ AbsoluteQuantitationMethod ]& aqm_list) nogil except +
        void store(const String& filename, libcpp_vector[ AbsoluteQuantitationMethod ]& aqm_list) nogil except +



from String cimport *
from Types cimport *
from DateTime cimport *
from Param cimport *

cdef extern from "<OpenMS/CONCEPT/UniqueIdGenerator.h>" namespace "OpenMS":

    cdef cppclass UniqueIdGenerator:

        # void setSeed(DateTime & time)

        UInt64 getUniqueId() nogil except +

        # Initializes random generator using the given DateTime instead of DateTime::now().  This is intended for debugging and testing.
        void setSeed(UInt64) nogil except +

        # Returns the seed
        UInt64 getSeed() nogil except +




from AbsoluteQuantitationStandards cimport *
from String cimport *
from Types cimport *

cdef extern from "<OpenMS/FORMAT/AbsoluteQuantitationStandardsFile.h>" namespace "OpenMS":

    cdef cppclass AbsoluteQuantitationStandardsFile:

        AbsoluteQuantitationStandardsFile() nogil except +
        AbsoluteQuantitationStandardsFile(AbsoluteQuantitationStandardsFile &) nogil except +

        void load(const String& filename, libcpp_vector[AQS_runConcentration]& run_concentrations) nogil except +





    def rip(self, dict ripped, list proteins, list peptides):
 
  
        # Input types:
        # ripped   :  libcpp_map[String, libcpp_pair[ libcpp_vector[ProteinIdentification], libcpp_vector[PeptideIdentification]]]
        # proteins :  libcpp_vector[ProteinIdentification] 
        # peptides :  libcpp_vector[PeptideIdentification] 

        assert isinstance(proteins, list) and all(isinstance(li, ProteinIdentification) for li in proteins), 'arg proteins wrong type'
        cdef libcpp_vector[_ProteinIdentification] * c_protein_vec = new libcpp_vector[_ProteinIdentification]()
        cdef ProteinIdentification item1
        for item1 in proteins:
           c_protein_vec.push_back(deref(item1.inst.get()))
 
        assert isinstance(peptides, list) and all(isinstance(li, PeptideIdentification) for li in peptides), 'arg peptides wrong type'
        cdef libcpp_vector[_PeptideIdentification] * c_peptide_vec = new libcpp_vector[_PeptideIdentification]()
        cdef PeptideIdentification item0
        for item0 in peptides:
           c_peptide_vec.push_back(deref(item0.inst.get()))
 
        # dict -> pair ( list, list ) )
        assert isinstance(ripped, dict) and all(isinstance(li, list) and len(li) == 2 and all( 
                isinstance(pair_element[0], list) and isinstance(pair_element[1], list) and
                all(isinstance(proteinid, ProteinIdentification) for proteinid in pair_element[0]) and 
                all(isinstance(peptideid, PeptideIdentification) for peptideid in pair_element[1]) 
            for pair_element in li) 
        for li in ripped), 'arg proteins wrong type'

        cdef libcpp_map[_String, libcpp_pair[ libcpp_vector[_ProteinIdentification], libcpp_vector[_PeptideIdentification]]] c_ripped
        # declaration for the loop 
        cdef libcpp_vector[_ProteinIdentification] * c_protein_vec_inner = new libcpp_vector[_ProteinIdentification]()
        cdef ProteinIdentification i_item1
        cdef libcpp_vector[_PeptideIdentification] * c_peptide_vec_inner = new libcpp_vector[_PeptideIdentification]()
        cdef PeptideIdentification i_item0
        cdef libcpp_pair[ libcpp_vector[_ProteinIdentification], libcpp_vector[_PeptideIdentification]] * aPair
        for k,v in ripped.iteritems():
 
            c_protein_vec_inner.clear()
            c_peptide_vec_inner.clear()
 
            prot_vec = v[0]
            assert isinstance(prot_vec, list) and all(isinstance(li, ProteinIdentification) for li in prot_vec), 'arg proteins wrong type'
            for i_item1 in prot_vec:
               c_protein_vec_inner.push_back(deref(i_item1.inst.get()))
 
            pep_vec = v[1]
            assert isinstance(pep_vec, list) and all(isinstance(li, PeptideIdentification) for li in pep_vec), 'arg peptides wrong type'
            for i_item0 in pep_vec:
               c_peptide_vec_inner.push_back(deref(i_item0.inst.get()))
 
            aPair = new libcpp_pair[ libcpp_vector[_ProteinIdentification], libcpp_vector[_PeptideIdentification]]( deref(c_protein_vec_inner), deref(c_peptide_vec_inner) )
 
            assert isinstance(k, bytes), 'arg key in label_identifiers wrong type'
            assert isinstance(v, float), 'arg value in label_identifiers wrong type'
            c_ripped[ _String(<char *>k) ] = deref(aPair)
 
            del aPair
 
        #
        ## Make the function call
        # 
        self.inst.get().rip(c_ripped, deref(c_protein_vec), deref(c_peptide_vec))

        #
        ## Get the data back from C++
        #
        replace = dict()
        cdef libcpp_map[_String, libcpp_pair[ libcpp_vector[_ProteinIdentification], libcpp_vector[_PeptideIdentification]]].iterator it_ripped = c_ripped.begin()
        cdef libcpp_pair[ libcpp_vector[_ProteinIdentification], libcpp_vector[_PeptideIdentification]] anotherPair
        cdef libcpp_vector[_ProteinIdentification] another_c_protein_vec_inner 
        cdef libcpp_vector[_PeptideIdentification] another_c_peptide_vec_inner 
        cdef libcpp_vector[_ProteinIdentification].iterator it_prot
        cdef libcpp_vector[_PeptideIdentification].iterator it_pep
        cdef ProteinIdentification item_py_result_prot
        cdef PeptideIdentification item_py_result_pep
        while it_ripped != c_ripped.end():
            # get the pair and the two vectors
            anotherPair = deref(it_ripped).second
            another_c_protein_vec_inner = anotherPair.first
            another_c_peptide_vec_inner = anotherPair.second

            replace_inner_protein = []
            it_prot = another_c_protein_vec_inner.begin()
            while it_prot != another_c_protein_vec_inner.end():
                item_py_result_prot = ProteinIdentification.__new__(ProteinIdentification)   
                item_py_result_prot.inst = shared_ptr[_ProteinIdentification](new _ProteinIdentification(deref(it_prot)))
                replace_inner_protein.append(item_py_result_prot)
                inc(it_prot)

            replace_inner_peptide = []
            it_pep = another_c_peptide_vec_inner.begin()
            while it_pep != another_c_peptide_vec_inner.end():
                item_py_result_pep = PeptideIdentification.__new__(PeptideIdentification)   
                item_py_result_pep.inst = shared_ptr[_PeptideIdentification](new _PeptideIdentification(deref(it_pep)))
                replace_inner_peptide.append(item_py_result_pep)
                inc(it_pep)

            replace[ <libcpp_string>deref(it_ripped).first ]  = [ replace_inner_protein, replace_inner_peptide] 
            inc(it_ripped)
        ripped.clear()
        ripped.update(replace)

        del c_peptide_vec
        del c_protein_vec
        del c_protein_vec_inner
        del c_peptide_vec_inner
 




from libcpp.vector cimport vector as libcpp_vector

from ChromatogramPeak cimport *
from DefaultParamHandler cimport *
from MSExperiment cimport *
from Peak1D cimport *
from ProgressLogger cimport *
from TransformationDescription cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/MapAlignmentAlgorithmSpectrumAlignment.h>" namespace "OpenMS":
    
    cdef cppclass MapAlignmentAlgorithmSpectrumAlignment(DefaultParamHandler, ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        MapAlignmentAlgorithmSpectrumAlignment() nogil except +
 
        void align(libcpp_vector[MSExperiment]&, libcpp_vector[TransformationDescription]&) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp.set cimport set as libcpp_set
from libcpp cimport bool

from String cimport *
from Map cimport *
from StringList cimport *
from CVMappings cimport *
from DataValue cimport *

cdef extern from "<OpenMS/FORMAT/ControlledVocabulary.h>" namespace "OpenMS":

    cdef cppclass ControlledVocabulary:

        ControlledVocabulary() nogil except +
        ControlledVocabulary(ControlledVocabulary & voc) nogil except +

        # Returns the CV name (set in the load method)
        String name() nogil except +

        void loadFromOBO(String name, String filename) nogil except +

        # Returns true if the term is in the CV. Returns false otherwise.
        bool exists(String id) nogil except +

        # Returns true if a term with the given name is in the CV. Returns false otherwise.
        bool hasTermWithName(String name) nogil except +

        CVTerm_ControlledVocabulary getTerm(String id) nogil except +

        CVTerm_ControlledVocabulary getTermByName(String name, String desc) nogil except +

        # returns all the terms stored in the CV
        # TODO OpenMS Map type
        # Map[String, CVTerm_ControlledVocabulary] getTerms() nogil except +

        void getAllChildTerms(libcpp_set[String] terms, String parent) nogil except +

        bool isChildOf(String child, String parent) nogil except +

cdef extern from "<OpenMS/FORMAT/ControlledVocabulary.h>" namespace "OpenMS::ControlledVocabulary":

    cdef cppclass CVTerm_ControlledVocabulary "OpenMS::ControlledVocabulary::CVTerm":

      String name  #< Text name
      String id  #< Identifier
      libcpp_set[String] parents  #< The parent IDs
      libcpp_set[String] children  #< The child IDs
      bool obsolete  #< Flag that indicates of the term is obsolete
      String description  #< Term description
      StringList synonyms  #< List of synonyms
      StringList unparsed  #< Unparsed lines from the definition file
      XRefType_CVTerm_ControlledVocabulary xref_type  #< xref value-type for the CV-term
      StringList xref_binary  #< xref binary-data-type for the CV-term (list of all allowed data value types for the current binary data array)
      libcpp_set[String] units  #< unit accession ids, defined by relationship has units

      #Default constructor
      CVTerm_ControlledVocabulary() nogil except +
      CVTerm_ControlledVocabulary(CVTerm_ControlledVocabulary rhs) nogil except +

      String toXMLString(String ref, String value) nogil except +
      String toXMLString(String ref, DataValue value) nogil except +
      String getXRefTypeName(XRefType_CVTerm_ControlledVocabulary type) nogil except +
      bool isHigherBetterScore(CVTerm_ControlledVocabulary term) nogil except +

cdef extern from "<OpenMS/FORMAT/ControlledVocabulary.h>" namespace "OpenMS::ControlledVocabulary::CVTerm":

    # define xsd types allowed in cv term to specify their value-type
    cdef enum XRefType_CVTerm_ControlledVocabulary "OpenMS::ControlledVocabulary::CVTerm::XRefType":
        XSD_STRING = 0, # xsd:string A string
        XSD_INTEGER, # xsd:integer Any integer
        XSD_DECIMAL, # xsd:decimal Any real number
        XSD_NEGATIVE_INTEGER, # xsd:negativeInteger Any negative integer
        XSD_POSITIVE_INTEGER, # xsd:positiveInteger Any integer ] 0
        XSD_NON_NEGATIVE_INTEGER, # xsd:nonNegativeInteger Any integer ]= 0
        XSD_NON_POSITIVE_INTEGER, # xsd:nonPositiveInteger Any integer < 0
        XSD_BOOLEAN, # xsd:boolean True or false
        XSD_DATE, # xsd:date An XML-Schema date
        XSD_ANYURI, # xsd:anyURI uniform resource identifier
        NONE




from Types cimport *
from libcpp.pair cimport pair as libcpp_pair
from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from MSSpectrum cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/COMPARISON/SPECTRA/SpectrumAlignment.h>" namespace "OpenMS":
    
    cdef cppclass SpectrumAlignment(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        SpectrumAlignment() nogil except +
        SpectrumAlignment(SpectrumAlignment) nogil except +

        void getSpectrumAlignment(libcpp_vector[ libcpp_pair[ Size, Size ] ] & alignment, MSSpectrum & s1, MSSpectrum & s2) nogil except +  # wrap-ignore



from Types cimport *
from Param cimport *
from String cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/TransformationModel.h>" namespace "OpenMS":

    cdef cppclass TM_DataPoint "OpenMS::TransformationModel::DataPoint":
        TM_DataPoint() nogil except +
        TM_DataPoint(double, double) nogil except +
        TM_DataPoint(double, double, const String&) nogil except +
        TM_DataPoint(TM_DataPoint) nogil except + #wrap-ignore
        bool operator<(TM_DataPoint&) nogil except +
        bool operator==(TM_DataPoint&) nogil except +

        double first
        double second
        String note


    cdef cppclass TransformationModel:
        # wrap-ignore
        # no-pxd-import

        TransformationModel()  nogil except +
        TransformationModel(TransformationModel, Param) nogil except + # wrap-ignore

        Param getParameters() nogil except +

        # double evaluate(double value) nogil except +
        # void getDefaultParameters(Param & params) nogil except +
        
        void weightData(libcpp_vector[TM_DataPoint]& data) nogil except +
        void weightData(libcpp_vector[TM_DataPoint]& data) nogil except +
        bool checkValidWeight(const String& weight, libcpp_vector[String]& valid_weights) nogil except +
        double weightDatum(double& datum, const String& weight) nogil except +
        double unWeightDatum(double& datum, const String& weight) nogil except +
        libcpp_vector[ String ] getValidXWeights() nogil except +
        libcpp_vector[ String ] getValidYWeights() nogil except +

        void unWeightData(libcpp_vector[TM_DataPoint] & data) nogil except +
        double checkDatumRange(const double & datum, const double & datum_min, const double & datum_max) nogil except +




from Types cimport *
from String cimport *
from ConvexHull2D cimport *
from libcpp.pair cimport pair as libcpp_pair
from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/KERNEL/MassTrace.h>" namespace "OpenMS":

    cdef cppclass Kernel_MassTrace "OpenMS::MassTrace":

        Kernel_MassTrace()  nogil except +
        Kernel_MassTrace(Kernel_MassTrace &) nogil except +

        # public members
        double fwhm_mz_avg

        Size getSize() nogil except +
        String getLabel() nogil except +
        void setLabel(String label) nogil except +

        double getCentroidMZ() nogil except +
        double getCentroidRT() nogil except +
        double getCentroidSD() nogil except +
        double getFWHM() nogil except +
        double getTraceLength() nogil except +
        libcpp_pair[Size,Size] getFWHMborders() nogil except +
        libcpp_vector[double] getSmoothedIntensities() nogil except +
        double getAverageMS1CycleTime() nogil except +

        double computeSmoothedPeakArea() nogil except +
        double computePeakArea() nogil except +
        Size findMaxByIntPeak(bool) nogil except +
        Size estimateFWHM(bool) nogil except +
        double computeFwhmArea() nogil except +
        double computeFwhmAreaSmooth() nogil except +
        # double computeFwhmAreaRobust() nogil except +
        # double computeFwhmAreaSmoothRobust() nogil except +
        double getIntensity(bool) nogil except +
        double getMaxIntensity(bool) nogil except +

        ConvexHull2D getConvexhull() nogil except +

        void setCentroidSD(double &tmp_sd) nogil except +
        void setSmoothedIntensities(libcpp_vector[ double ] &db_vec) nogil except +
        void updateSmoothedMaxRT() nogil except +
        void updateWeightedMeanRT() nogil except +
        void updateSmoothedWeightedMeanRT() nogil except +
        void updateMedianRT() nogil except +
        void updateMedianMZ() nogil except +
        void updateMeanMZ() nogil except +
        void updateWeightedMeanMZ() nogil except +
        void updateWeightedMZsd() nogil except +

        void setQuantMethod(MT_QUANTMETHOD method) nogil except +
        MT_QUANTMETHOD getQuantMethod() nogil except +
        
cdef extern from "<OpenMS/KERNEL/MassTrace.h>" namespace "OpenMS::MassTrace":

    cdef enum MT_QUANTMETHOD:
        MT_QUANT_AREA,
        MT_QUANT_MEDIAN,
        SIZE_OF_MT_QUANTMETHOD




import pyopenms
import numpy

def _test_container(c, i, append_method="push_back"):
    # did raise segfault because of missing index check in __getitem__
    # should be resolved by implementing a proper iterator:
    s = c()
    assert list(s) == []
    try:
        s[0]
    except IndexError:
        pass
    else:
        assert False, "no exception risen"

    exec("s.%s(i())" % append_method)
    (item,) = s # test iterator

    s[0]  # test getattr

    try:
        s[1]
    except IndexError:
        pass
    else:
        assert False, "no exception risen"

def testContainers():
    _test_container(pyopenms.MSSpectrum, pyopenms.Peak1D)
    _test_container(pyopenms.MSExperiment, pyopenms.MSSpectrum, "addSpectrum")
    _test_container(pyopenms.FeatureMap, pyopenms.Feature)


def testParamEntry():
    # as ParamEntry::isValid takes "String &" as input argument, which
    # can not be implemened by a Python string, here no automatic
    # conversion from a basestring should happen:
    p = pyopenms.ParamEntry()
    message = pyopenms.String()
    assert p.isValid(message)
    assert message.c_str() == b""

def testConvexHull2D():
    h = pyopenms.ConvexHull2D()

    points = numpy.arange(10.0, dtype=numpy.float32).reshape(-1, 2)

    h.setHullPoints(points)
    p = h.getHullPoints()
    assert numpy.linalg.norm(p - points) < 1e-6

    h.expandToBoundingBox()
    hullp = h.getHullPoints()
    assert set(hullp[:,0]) == set((0.0, 8.0))
    assert set(hullp[:,1]) == set((1.0, 9.0))

    box = h.getBoundingBox()
    assert box.minPosition() == [ 0.0, 1.0]
    assert box.maxPosition() == [ 8.0, 9.0]

    h.addPoint([-10.0, -10.0])
    box = h.getBoundingBox()
    assert box.minPosition() == [ -10.0, -10.0]
    assert box.maxPosition() == [ 8.0, 9.0]
    h.expandToBoundingBox()
    hullp = h.getHullPoints()
    assert set(hullp[:,0]) == set((-10.0, 8.0))
    assert set(hullp[:,1]) == set((-10.0, 9.0))
    box = h.getBoundingBox()
    assert box.minPosition() == [ -10.0, -10.0]
    assert box.maxPosition() == [ 8.0, 9.0]




# part of the SpectraFilters
from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/FILTERING/TRANSFORMERS/WindowMower.h>" namespace "OpenMS":

    cdef cppclass WindowMower(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        WindowMower()            nogil except +
        WindowMower(WindowMower) nogil except + #wrap-ignore

        void filterPeakSpectrumForTopNInSlidingWindow(MSSpectrum & spectrum) nogil except + # wrap-doc:sliding window version (slower)
        void filterPeakSpectrumForTopNInJumpingWindow(MSSpectrum & spectrum) nogil except + # wrap-doc:jumping window version (faster)

        void filterPeakSpectrum(MSSpectrum & spec) nogil except +
        void filterPeakMap(MSExperiment & exp) nogil except +




from String cimport *
from Types cimport *

cdef extern from "<OpenMS/CONCEPT/UniqueIdInterface.h>" namespace "OpenMS":

    cdef cppclass UniqueIdInterface:
        # wrap-ignore
        # no-pxd-import

        UniqueIdInterface() nogil except +
        UniqueIdInterface(UniqueIdInterface) nogil except +

        Size getUniqueId() nogil except +
        Size clearUniqueId() nogil except +
        void swap(UniqueIdInterface) nogil except + # wrap-ignore
        Size hasValidUniqueId() nogil except +
        Size hasInvalidUniqueId() nogil except +
        void setUniqueId(UInt64 rhs) nogil except +
        Size ensureUniqueId() nogil except +
        
        # overloading Cython Issue
        # TODO: Mismatch between C++ return type ([u'Size']) and Python return type (['void']) in function public setUniqueId:
        # Size setUniqueId() nogil except +
        # void setUniqueId(const String & rhs) nogil except +

        bool isValid(UInt64 unique_id) nogil except +

cdef extern from "<OpenMS/CONCEPT/UniqueIdInterface.h>" namespace "OpenMS::UniqueIdInterface":

        Size setUniqueId() # wrap-ignore







from Types cimport *
from libcpp cimport bool
from libcpp.pair cimport pair as libcpp_pair
from DefaultParamHandler cimport *
from BaseFeature cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/FeatureDistance.h>" namespace "OpenMS":
    
    cdef cppclass FeatureDistance(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        FeatureDistance(FeatureDistance) nogil except + #wrap-ignore
        # TODO  is static const -> no setters please
        # double infinity
        FeatureDistance(double max_intensity, bool force_constraints) nogil except +
        # libcpp_pair[ bool, double ] operator()(BaseFeature & left, BaseFeature & right) nogil except +




from libcpp cimport bool
from String cimport *
from Types cimport *
# from SampleTreatment cimport *

cdef extern from "<OpenMS/METADATA/Digestion.h>" namespace "OpenMS":
    
    cdef cppclass Digestion "OpenMS::Digestion":
        Digestion() nogil except +
        Digestion(Digestion) nogil except +
        # bool operator==(SampleTreatment &rhs) nogil except +
        # SampleTreatment * clone() nogil except +
        String  getEnzyme() nogil except +
        void setEnzyme(const String& enzyme) nogil except +
        double getDigestionTime() nogil except +
        void setDigestionTime(double digestion_time) nogil except +
        double getTemperature() nogil except +
        void setTemperature(double temperature) nogil except +
        double getPh() nogil except +
        void setPh(double ph) nogil except +




from ProgressLogger cimport *
from DefaultParamHandler cimport *
from MSExperiment cimport *
from MSSpectrum cimport *
from MSChromatogram cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/FILTERING/SMOOTHING/GaussFilter.h>" namespace "OpenMS":

    cdef cppclass GaussFilter(DefaultParamHandler,ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        GaussFilter() nogil except +
        GaussFilter(GaussFilter) nogil except +

        void filter(MSSpectrum & spectrum) nogil except +
        void filter(MSChromatogram & chromatogram) nogil except +
        void filterExperiment(MSExperiment & exp) nogil except +




from Types cimport *
from libcpp.map cimport map as libcpp_map
from libcpp.vector cimport vector as libcpp_vector
from PeptideIdentification cimport *
from AASequence cimport *
from XMLFile cimport *
from SpectrumMetaDataLookup cimport *
from Map cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/FORMAT/MascotXMLFile.h>" namespace "OpenMS":
    
    cdef cppclass MascotXMLFile(XMLFile) :
        # wrap-inherits:
        #  XMLFile
        MascotXMLFile() nogil except +
        MascotXMLFile(MascotXMLFile) nogil except + #wrap-ignore

        void load(const String & filename,
                  ProteinIdentification & protein_identification,
                  libcpp_vector[ PeptideIdentification ] & id_data,
                  SpectrumMetaDataLookup & rt_mapping) nogil except +

        # TODO fix
        # void load(const String & filename,
        #           ProteinIdentification & protein_identification,
        #           libcpp_vector[ PeptideIdentification ] & id_data,
        #           libcpp_map[ String, libcpp_vector[ AASequence ] ] & peptides,
        #           SpectrumMetaDataLookup & rt_mapping) nogil except +

        void initializeLookup(SpectrumMetaDataLookup & lookup, MSExperiment& experiment, const String & scan_regex) nogil except +




from libcpp cimport bool
from String cimport *
from Types cimport *

cdef extern from "<OpenMS/CONCEPT/VersionInfo.h>" namespace "OpenMS":

    cdef cppclass VersionInfo:
        pass

cdef extern from "<OpenMS/CONCEPT/VersionInfo.h>" namespace "OpenMS::VersionInfo":

    cdef cppclass VersionDetails:
        Int version_major
        Int version_minor
        Int version_patch
        String pre_release_identifier

        VersionDetails() nogil except +
        VersionDetails(VersionDetails) nogil except +   #wrap-ignore
        bool operator<(VersionDetails) nogil except +
        bool operator==(VersionDetails) nogil except +
        bool operator>(VersionDetails) nogil except +

    VersionDetails getVersionStruct() nogil except +  #wrap-attach:VersionInfo
    String getVersion()  nogil except +  #wrap-attach:VersionInfo
    String getTime()     nogil except +  #wrap-attach:VersionInfo
    String getRevision() nogil except +  #wrap-attach:VersionInfo
    String getBranch()   nogil except +  #wrap-attach:VersionInfo

cdef extern from "<OpenMS/CONCEPT/VersionInfo.h>" namespace "OpenMS::VersionInfo::VersionDetails":

    VersionDetails create(String) #wrap-attach:VersionDetails




from Types cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from libcpp.map cimport map as libcpp_map
from String cimport *

cdef extern from "<OpenMS/FORMAT/InspectInfile.h>" namespace "OpenMS":

    cdef cppclass InspectInfile "OpenMS::InspectInfile":

        InspectInfile() nogil except +
        InspectInfile(InspectInfile) nogil except +

        bool operator==(InspectInfile & inspect_infile) nogil except +
        void store(const String & filename) nogil except +
        void handlePTMs(const String & modification_line, const String & modifications_filename, bool monoisotopic) nogil except +
        String  getSpectra() nogil except +
        void setSpectra(const String & spectra) nogil except +
        String  getDb() nogil except +
        void setDb(const String & db) nogil except +
        String  getEnzyme() nogil except +
        void setEnzyme(const String & enzyme) nogil except +
        Int getModificationsPerPeptide() nogil except +
        void setModificationsPerPeptide(Int modifications_per_peptide) nogil except +
        UInt getBlind() nogil except +
        void setBlind(UInt blind) nogil except +
        float getMaxPTMsize() nogil except +
        void setMaxPTMsize(float maxptmsize) nogil except +
        float getPrecursorMassTolerance() nogil except +
        void setPrecursorMassTolerance(float precursor_mass_tolerance) nogil except +
        float getPeakMassTolerance() nogil except +
        void setPeakMassTolerance(float peak_mass_tolerance) nogil except +
        UInt getMulticharge() nogil except +
        void setMulticharge(UInt multicharge) nogil except +
        String  getInstrument() nogil except +
        void setInstrument(const String & instrument) nogil except +
        Int getTagCount() nogil except +
        void setTagCount(Int TagCount) nogil except +

        libcpp_map[ String, libcpp_vector[ String ] ]  getModifications() nogil except + # wrap-ignore




from libcpp.map cimport map as libcpp_map
from String cimport *
from libcpp.string cimport string as libcpp_string
# from MzTabSmallMoleculeSectionData.string cimport string as libcpp_string

cdef extern from "<OpenMS/FORMAT/MzTab.h>" namespace "OpenMS":

    cdef cppclass MzTab:

        MzTab() nogil except +
        MzTab(MzTab) nogil except + # wrap-ignore

        # autowrap cannot handle wrapped classes as keys in std::map<> (this
        # would be std::map<String, ...> and does not work)
        # TODO STL map with wrapped key

        # libcpp_map[libcpp_string, MzTabUnitIdMetaData] getMetaData() const
        # MzTabProteinSectionData getProteinSectionData() const
        # MzTabPeptideSectionData getPeptideSectionData() const
        # MzTabSmallMoleculeSectionData getSmallMoleculeSectionData() const




from ConvexHull2D cimport *
from DefaultParamHandler cimport *
from MSChromatogram cimport *
from MSSpectrum cimport *
from Types cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/PeakIntegrator.h>" namespace "OpenMS":

    cdef cppclass PeakIntegrator(DefaultParamHandler):
        # wrap-inherits:
        #  DefaultParamHandler

        PeakIntegrator() nogil except +
        PeakIntegrator(PeakIntegrator) nogil except +

        void getDefaultParameters(Param) nogil except +

        PI_PeakArea integratePeak(MSChromatogram& chromatogram, double left, double right) nogil except +
        PI_PeakArea integratePeak(MSSpectrum& spectrum, double left, double right) nogil except +

        PI_PeakBackground estimateBackground(MSChromatogram& chromatogram, double left, double right, double peak_apex_pos) nogil except +
        PI_PeakBackground estimateBackground(MSSpectrum& spectrum, double left, double right, double peak_apex_pos) nogil except +

        PI_PeakShapeMetrics calculatePeakShapeMetrics(MSChromatogram& chromatogram, double left, double right, double peak_height, double peak_apex_pos) nogil except +
        PI_PeakShapeMetrics calculatePeakShapeMetrics(MSSpectrum& spectrum, double left, double right, double peak_height, double peak_apex_pos) nogil except +


cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/PeakIntegrator.h>" namespace "OpenMS::PeakIntegrator":

    cdef cppclass PI_PeakArea "OpenMS::PeakIntegrator::PeakArea":

        PI_PeakArea() nogil except +
        PI_PeakArea(PI_PeakArea) nogil except +

        double area
        double height
        double apex_pos
        libcpp_vector[DPosition2] hull_points

    cdef cppclass PI_PeakBackground "OpenMS::PeakIntegrator::PeakBackground":

        PI_PeakBackground() nogil except +
        PI_PeakBackground(PI_PeakBackground) nogil except +

        double area
        double height

    cdef cppclass PI_PeakShapeMetrics "OpenMS::PeakIntegrator::PeakShapeMetrics":

        PI_PeakShapeMetrics() nogil except +
        PI_PeakShapeMetrics(PI_PeakShapeMetrics) nogil except +

        double width_at_5
        double width_at_10
        double width_at_50
        double start_position_at_5
        double start_position_at_10
        double start_position_at_50
        double end_position_at_5
        double end_position_at_10
        double end_position_at_50
        double total_width
        double tailing_factor
        double asymmetry_factor
        double slope_of_baseline
        double baseline_delta_2_height
        Int points_across_baseline
        Int points_across_half_height



from MSSpectrum cimport *
from FeatureMap cimport *
from ConsensusMap cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from Param cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/ANALYSIS/DECHARGING/FeatureDeconvolution.h>" namespace "OpenMS":

    cdef cppclass FeatureDeconvolution(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        FeatureDeconvolution()                  nogil except +
        FeatureDeconvolution(FeatureDeconvolution)   nogil except + #wrap-ignore

        void compute(FeatureMap & input, FeatureMap & output, ConsensusMap & cmap1, ConsensusMap & cmap2) nogil except +

cdef extern from "<OpenMS/ANALYSIS/DECHARGING/FeatureDeconvolution.h>" namespace "OpenMS::FeatureDeconvolution":
    
    cdef enum CHARGEMODE_FD "OpenMS::FeatureDeconvolution::CHARGEMODE":
        #wrap-attach:
        #    FeatureDeconvolution
        # todo -- are these really unique ??  prob not! all become __CHARGEMODE
        # TODO wrap-instances:
        #    CHARGEMODE := CHARGEMODE_FD
        QFROMFEATURE
        QHEURISTIC
        QALL




from MRMFeaturePicker cimport *
from String cimport *
from Types cimport *

cdef extern from "<OpenMS/FORMAT/MRMFeaturePickerFile.h>" namespace "OpenMS":

    cdef cppclass MRMFeaturePickerFile:

        MRMFeaturePickerFile() nogil except +
        MRMFeaturePickerFile(MRMFeaturePickerFile &) nogil except +

        void load(const String& filename, libcpp_vector[MRMFP_ComponentParams]& cp_list, libcpp_vector[MRMFP_ComponentGroupParams]& cgp_list) nogil except +



from Types cimport *
from libcpp cimport bool
from MSSpectrum cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/ANALYSIS/RNPXL/HyperScore.h>" namespace "OpenMS":
    
    cdef cppclass HyperScore "OpenMS::HyperScore":
        HyperScore() nogil except + 
        HyperScore(HyperScore) nogil except + #wrap-ignore

        double compute(double fragment_mass_tolerance, 
                       bool fragment_mass_tolerance_unit_ppm,
                       MSSpectrum & exp_spectrum, MSSpectrum & theo_spectrum) nogil except +




from Types cimport *
from libcpp cimport bool
from Map cimport *
from String cimport *

cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/MassDecomposition.h>" namespace "OpenMS":
    
    cdef cppclass MassDecomposition "OpenMS::MassDecomposition":
        MassDecomposition() nogil except +
        MassDecomposition(MassDecomposition) nogil except +
        MassDecomposition(const String & deco) nogil except +
        # MassDecomposition  operator+=(MassDecomposition & d) nogil except +
        String toString() nogil except +
        String toExpandedString() nogil except +
        # MassDecomposition operator+(MassDecomposition & rhs) nogil except +
        Size getNumberOfMaxAA() nogil except +
        # bool operator<(MassDecomposition & rhs) nogil except +
        # bool operator==(const String & deco) nogil except +
        bool containsTag(const String & tag) nogil except +
        bool compatible(MassDecomposition & deco) nogil except +




from Types cimport *
from libcpp.pair cimport pair as libcpp_pair
from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from ProgressLogger cimport *
from ConsensusMap cimport *
from MSExperiment cimport *

# typedef std::vector<std::pair<double, double> > MasstracePointsType;

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MasstraceCorrelator.h>" namespace "OpenMS":
    
    ctypedef libcpp_vector[libcpp_pair[double,double ] ] MasstracePointsType

    cdef cppclass MasstraceCorrelator(DefaultParamHandler,ProgressLogger) :
        # wrap-inherits:
        #  DefaultParamHandler
        #  ProgressLogger
        MasstraceCorrelator() nogil except +
        MasstraceCorrelator(MasstraceCorrelator) nogil except + #wrap-ignore
        void createPseudoSpectra(const ConsensusMap & map_,
                                 MSExperiment & pseudo_spectra,
                                 Size min_peak_nr,
                                 double min_correlation,
                                 int max_lag,
                                 double max_rt_apex_difference) nogil except +

        # void scoreHullpoints(const MasstracePointsType & hull_points1,
        #                      const MasstracePointsType & hull_points2,
        #                      int lag,
        #                      double lag_intensity,
        #                      double pearson_score,
        #                      double min_corr,
        #                      int max_lag,
        #                      double mindiff) nogil except +

        # void createConsensusMapCache(const ConsensusMap & map_,
        #                              libcpp_vector[ MasstracePointsType ] & feature_points,
        #                              libcpp_vector[ libcpp_pair[ double, double ] ] & max_intensities,
        #                              libcpp_vector[ double ] & rt_cache) nogil except +







    def getModifications(self):
        _r = self.inst.get().getModifications()
        py_result = dict()
        cdef Map[_String, libcpp_vector[_String]].iterator outer_it = _r.begin()
        cdef libcpp_vector[_String].iterator inner_it
        cdef String item_0
        cdef bytes inner_key
        cdef list inner_values
        while outer_it != _r.end():
           inner_key = deref(outer_it).first.c_str()
           inner_values = []
           inner_it = deref(outer_it).second.begin()
           while inner_it != deref(outer_it).second.end():
               # item_0 = CVTerm.__new__(CVTerm)
               # item_0.inst = shared_ptr[_CVTerm](new _CVTerm(deref(inner_it)))
               inner_values.append(  deref(inner_it).c_str() )
               inc(inner_it)
           py_result[inner_key] = inner_values
           inc(outer_it)
        return py_result




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from DefaultParamHandler cimport *
from Peak1D cimport *
from MSSpectrum cimport *

cdef extern from "<OpenMS/ANALYSIS/MRM/MRMFragmentSelection.h>" namespace "OpenMS":
    
    cdef cppclass MRMFragmentSelection(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        MRMFragmentSelection() nogil except +
        MRMFragmentSelection(MRMFragmentSelection) nogil except +
        void selectFragments(libcpp_vector[ Peak1D ] & selected_peaks, MSSpectrum & spec) nogil except +




from Types cimport *
from Param cimport *

from TransformationModel cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/TransformationModelInterpolated.h>" namespace "OpenMS":

    cdef cppclass TransformationModelInterpolated(TransformationModel):
        # wrap-inherits:
        #   TransformationModel

        TransformationModelInterpolated(TransformationModelInterpolated) nogil except + #wrap-ignore
        TransformationModelInterpolated(libcpp_vector[TM_DataPoint]& data, Param& params) nogil except +

        void getDefaultParameters(Param &)
        double evaluate(double value) nogil except +

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/TransformationModelInterpolated.h>" namespace "OpenMS:TransformationModelInterpolated":

    cdef cppclass Interpolator:
        # wrap-ignore
        # ABSTRACT
        # no-pxd-import

      void init(libcpp_vector[double] x, libcpp_vector[double] y)

      double eval(double x)



from libcpp.vector cimport vector as libcpp_vector
from libcpp.string cimport string as libcpp_string
from Types cimport *
from libcpp cimport bool
from MSSpectrum cimport *

cdef extern from "<OpenMS/COMPARISON/SPECTRA/BinnedSpectrum.h>" namespace "OpenMS":

    cdef cppclass BinnedSpectrum:
        BinnedSpectrum() nogil except + #wrap-ignore
        BinnedSpectrum(BinnedSpectrum &) nogil except + #wrap-ignore
        BinnedSpectrum(MSSpectrum, float size, bool unit_ppm, UInt spread, float offset) nogil except +

        bool operator==(BinnedSpectrum & rhs) nogil except +
        bool operator!=(BinnedSpectrum & rhs) nogil except +

        float getBinSize() nogil except +
        UInt getBinSpread() nogil except +
        UInt getBinIndex(float mz) nogil except +
        float getBinLowerMZ(size_t i) nogil except +
        float getBinIntensity(double mz) nogil except +

        libcpp_vector[Precursor] getPrecursors() nogil except +

        bool isCompatible(BinnedSpectrum & a, BinnedSpectrum & b) nogil except +
        float getOffset() nogil except +




from Types cimport *
from IsobaricQuantitationMethod cimport *
from ItraqFourPlexQuantitationMethod cimport *
from ItraqEightPlexQuantitationMethod cimport *
from TMTSixPlexQuantitationMethod cimport *
from TMTTenPlexQuantitationMethod cimport *
from ConsensusMap cimport *
from IsobaricQuantifierStatistics cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/IsobaricIsotopeCorrector.h>" namespace "OpenMS":

    cdef cppclass IsobaricIsotopeCorrector:

        IsobaricIsotopeCorrector() nogil except +
        IsobaricIsotopeCorrector(IsobaricIsotopeCorrector) nogil except + # wrap-ignore

        # IsobaricQuantifierStatistics correctIsotopicImpurities(ConsensusMap & consensus_map_in,
        #                                                        ConsensusMap & consensus_map_out,
        #                                                        IsobaricQuantitationMethod * quant_method) nogil except + # wrap-ignore

        IsobaricQuantifierStatistics correctIsotopicImpurities(ConsensusMap & consensus_map_in,
                                                               ConsensusMap & consensus_map_out,
                                                               ItraqEightPlexQuantitationMethod * quant_method) nogil except +
        IsobaricQuantifierStatistics correctIsotopicImpurities(ConsensusMap & consensus_map_in,
                                                               ConsensusMap & consensus_map_out,
                                                               ItraqFourPlexQuantitationMethod * quant_method) nogil except +
        IsobaricQuantifierStatistics correctIsotopicImpurities(ConsensusMap & consensus_map_in,
                                                               ConsensusMap & consensus_map_out,
                                                               TMTSixPlexQuantitationMethod * quant_method) nogil except +
        IsobaricQuantifierStatistics correctIsotopicImpurities(ConsensusMap & consensus_map_in,
                                                               ConsensusMap & consensus_map_out,
                                                               TMTTenPlexQuantitationMethod * quant_method) nogil except +




from Types cimport *
from libcpp.pair cimport pair as libcpp_pair
from libcpp.vector cimport vector as libcpp_vector
from XMLFile cimport *
from PeptideIdentification cimport *
from ProteinIdentification cimport *
from CrossLinkSpectrumMatch cimport *
from PreprocessedPairSpectra cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/FORMAT/XQuestResultXMLFile.h>" namespace "OpenMS":

    cdef cppclass XQuestResultXMLFile(XMLFile) :
        # wrap-inherits:
        #  XMLFile

        XQuestResultXMLFile() nogil except +
        XQuestResultXMLFile(XQuestResultXMLFile) nogil except + #wrap-ignore

        void load(const String & filename,
                libcpp_vector[ PeptideIdentification ] & pep_ids,
                libcpp_vector[ ProteinIdentification ] & prot_ids) nogil except +

        void store(const String & filename,
                libcpp_vector[ ProteinIdentification ] & poid,
                libcpp_vector[ PeptideIdentification ] & peid) nogil except +

        int getNumberOfHits() nogil except +
        double getMinScore() nogil except +
        double getMaxScore() nogil except +

        void writeXQuestXMLSpec(const String& out_file, const String& base_name,
                                OPXL_PreprocessedPairSpectra preprocessed_pair_spectra,
                                libcpp_vector[ libcpp_pair[ size_t, size_t ] ] spectrum_pairs,
                                libcpp_vector[ libcpp_vector[ CrossLinkSpectrumMatch ] ] all_top_csms,
                                MSExperiment spectra) nogil except +

        void writeXQuestXMLSpec(const String& out_file, const String& base_name,
                                libcpp_vector[ libcpp_vector[ CrossLinkSpectrumMatch] ] all_top_csms,
                                MSExperiment spectra) nogil except +



from Types cimport *
from String cimport *

# TODO: support multiset
# typedef std::multiset<String> LabelSet;

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/MultiplexDeltaMasses.h>" namespace "OpenMS":
    
    cdef cppclass MultiplexDeltaMasses "OpenMS::MultiplexDeltaMasses":
        MultiplexDeltaMasses() nogil except +
        MultiplexDeltaMasses(MultiplexDeltaMasses) nogil except + #wrap-ignore

        MultiplexDeltaMasses(libcpp_vector[ MultiplexDeltaMasses_DeltaMass ] & dm) nogil except +
        libcpp_vector[ MultiplexDeltaMasses_DeltaMass ]  getDeltaMasses() nogil except +

        # String labelSetToString(LabelSet ls) nogil except +

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/MultiplexDeltaMasses.h>" namespace "OpenMS::MultiplexDeltaMasses":
    
    cdef cppclass MultiplexDeltaMasses_DeltaMass "OpenMS::MultiplexDeltaMasses::DeltaMass":

        MultiplexDeltaMasses_DeltaMass(MultiplexDeltaMasses_DeltaMass) nogil except + #wrap-ignore
        MultiplexDeltaMasses_DeltaMass(double dm, String l) nogil except +
        # DeltaMass(double dm, LabelSet ls);

        double delta_mass
        # LabelSet label_set
      
      



from Types cimport *
from smart_ptr cimport shared_ptr

cdef extern from "<OpenMS/OPENSWATHALGO/DATAACCESS/DataStructures.h>" namespace "OpenSwath":

  cdef cppclass OSBinaryDataArray:
        OSBinaryDataArray() nogil except +
        OSBinaryDataArray(OSBinaryDataArray) nogil except +
        libcpp_vector[double] data
        libcpp_string description
      
  ctypedef shared_ptr[OSBinaryDataArray] OSBinaryDataArrayPtr

  # See ../addons/OSSpectrum.pyx
  cdef cppclass OSSpectrum:
        OSSpectrum() nogil except +
        OSSpectrum(OSSpectrum) nogil except +
        OSBinaryDataArrayPtr getMZArray() #wrap-ignore
        OSBinaryDataArrayPtr getIntensityArray() #wrap-ignore
        # libcpp_vector[ BinaryDataArrayPtr ]  getDataArrays() nogil except +
        void setMZArray(OSBinaryDataArrayPtr data) #wrap-ignore
        void setIntensityArray(OSBinaryDataArrayPtr data) #wrap-ignore

  # boost::shared_ptr<OpenSwath::Spectrum> OpenSwath::SpectrumPtr;
  ctypedef shared_ptr[OSSpectrum] OSSpectrumPtr

  # See ../addons/OSChromatogram.pyx
  cdef cppclass OSChromatogram:
        OSChromatogram() nogil except +
        OSChromatogram(OSChromatogram) nogil except +
        OSBinaryDataArrayPtr getTimeArray() #wrap-ignore
        OSBinaryDataArrayPtr getIntensityArray() #wrap-ignore
        # libcpp_vector[ BinaryDataArrayPtr ]  getDataArrays() nogil except +
        void setTimeArray(OSBinaryDataArrayPtr data) #wrap-ignore
        void setIntensityArray(OSBinaryDataArrayPtr data) #wrap-ignore

  # boost::shared_ptr<OpenSwath::Chromatogram> OpenSwath::ChromatogramPtr;
  ctypedef shared_ptr[OSChromatogram] OSChromatogramPtr




from libc.string cimport const_char
from libcpp cimport bool
from libcpp.string cimport string as libcpp_string

cdef extern from "<OpenMS/DATASTRUCTURES/String.h>" namespace "OpenMS":

    # Note that all input will be encoded as UTF8 and stored as bytes inside
    # OpenMS::String (which is a wrapper around std::string). The data will be
    # decoded on its way back when calling "toString()" again as UTF8.
    # If you use another encoding, you will have a bad time!
    # Also, note that __str__ follows Python 2.x convention and returns bytes,
    # not unicode. Please use toString if you want to have encoded data.
    cdef cppclass String:
        # wrap-hash:
        #   c_str()

        String() nogil except +
        String(String) nogil except +  # wrap-ignore
        String(char *) nogil except + # wrap-ignore
        String(char *, size_t l) nogil except + # wrap-ignore
        String(str) nogil except + # wrap-ignore
        const_char * c_str() nogil except + # wrap-ignore

        # Creates a Python 2/3 unicode string (use this instead of c_str() if you
        # plan to use any non-ASCII code).
        toString(self) nogil except + # wrap-ignore

        bool operator==(String) nogil except +
        bool operator!=(String) nogil except +

        # Rather perform string operations in Python (you will have a bad time
        # with unicode strings otherwise).
        size_t length() nogil except + # wrap-ignore
        # libcpp_string operator[](int) nogil except + # wrap-upper-limit:length()

cdef extern from "<OpenMS/DATASTRUCTURES/String.h>" namespace "OpenMS::String":
    
    cdef enum QuotingMethod "OpenMS::String::QuotingMethod":
        NONE
        ESCAPE
        DOUBLE


#
#   def _init_2(self, str in_0 ):
#       assert isinstance(in_0, str), 'arg in_0 wrong type'
#   
#       print(b"have init 2 here")
#       # TODO catch encoding errors ...
#       py_byte_string = in_0.encode('UTF-8')
#       cdef char* c_string = py_byte_string
#       self.inst = shared_ptr[_String](new _String((<char *>c_string)))
#
#       # self.inst = shared_ptr[_String](new _String((<char *>in_0)))
#   
#   def __init__(self, *args):
#       if not args:
#            self._init_0(*args)
#       elif (len(args)==1) and (isinstance(args[0], bytes)):
#            self._init_1(*args)
#       elif (len(args)==1) and (isinstance(args[0], str)):
#            self._init_2(*args)
#       else:
#              print(b"aaaa")
#              raise Exception('can not handle type of %s' % (args,)) 
#



from libcpp cimport bool
from Types cimport *
from String cimport *
from ConsensusMap cimport *
from ExperimentalDesign cimport *
from StringList cimport *


cdef extern from "<OpenMS/FORMAT/MSstatsFile.h>" namespace "OpenMS":

    cdef cppclass MSstatsFile:

        MSstatsFile() nogil except +

        void storeLFQ(String & filename,
                      ConsensusMap & consensus_map,
                      ExperimentalDesign & design,
                      StringList & reannotate_filenames,
                      bool is_isotope_label_type,
                      String & bioreplicate,
                      String & condition,
                      String & retention_time_summarization_method) nogil except +

        void storeISO(String & filename,
                      ConsensusMap & consensus_map,
                      ExperimentalDesign & design,
                      StringList & reannotate_filenames,
                      String & bioreplicate,
                      String & condition,
                      String & mixture,
                      String & retention_time_summarization_method) nogil except +


from Types cimport *
from MSSpectrum cimport *
from MSChromatogram cimport *
from SplinePackage cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/FILTERING/DATAREDUCTION/SplineInterpolatedPeaks.h>" namespace "OpenMS":
    
    cdef cppclass SplineInterpolatedPeaks "OpenMS::SplineInterpolatedPeaks":

        SplineInterpolatedPeaks(libcpp_vector[double] mz, libcpp_vector[double] intensity) nogil except +

        SplineInterpolatedPeaks(MSSpectrum raw_spectrum) nogil except +

        SplineInterpolatedPeaks(MSChromatogram raw_chromatogram) nogil except +

        double getPosMin() nogil except +

        double getPosMax() nogil except +

        int size() nogil except +
        
        SplineSpectrum_Navigator getNavigator(double scaling) nogil except +


cdef extern from "<OpenMS/FILTERING/DATAREDUCTION/SplineInterpolatedPeaks.h>" namespace "OpenMS::SplineInterpolatedPeaks":
    
    cdef cppclass SplineSpectrum_Navigator "OpenMS::SplineInterpolatedPeaks::Navigator":
        
            SplineSpectrum_Navigator() nogil except + #wrap-ignore
            SplineSpectrum_Navigator(SplineSpectrum_Navigator) nogil except + #wrap-ignore
            
            SplineSpectrum_Navigator(libcpp_vector[SplinePackage]* packages, double posMin, double posMax, double scaling)  nogil except +

            double eval(double pos) nogil except +

            double getNextPos(double pos) nogil except +



from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from PeakShape cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/OptimizePick.h>" namespace "OpenMS":
    
    cdef cppclass OptimizePick "OpenMS::OptimizePick":
        OptimizePick() nogil except +
        OptimizePick(OptimizePick) nogil except + #wrap-ignore
        OptimizePick(OptimizationFunctions_PenaltyFactors penalties_, int max_iteration_) nogil except +
        OptimizationFunctions_PenaltyFactors getPenalties() nogil except +
        void setPenalties(OptimizationFunctions_PenaltyFactors penalties) nogil except +
        unsigned int  getNumberIterations() nogil except +
        void setNumberIterations(int max_iteration) nogil except +
        # void optimize(libcpp_vector[ PeakShape ] & peaks, OptimizePick_Data & data) nogil except +

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/OptimizePick.h>" namespace "OpenMS::OptimizationFunctions":
    
    cdef cppclass OptimizationFunctions_PenaltyFactors "OpenMS::OptimizationFunctions::PenaltyFactors":
        OptimizationFunctions_PenaltyFactors() nogil except +
        OptimizationFunctions_PenaltyFactors(OptimizationFunctions_PenaltyFactors) nogil except +
        double pos
        double lWidth
        double rWidth

cdef extern from "<OpenMS/TRANSFORMATIONS/RAW2PEAK/OptimizePick.h>" namespace "OpenMS::OptimizePick":
    
    cdef cppclass OptimizePick_Data "OpenMS::OptimizePick::Data":
        OptimizePick_Data(OptimizePick_Data) nogil except + #wrap-ignore
        libcpp_vector[ double ] positions
        libcpp_vector[ double ] signal

        # TODO STL attribute
        # OptimizationFunctions_PenaltyFactors penalties
        # libcpp_vector[ PeakShape ] peaks




from Types cimport *
from MassTrace cimport *
from Feature cimport *
from ConsensusFeature cimport *
from ConsensusMap cimport *
from FeatureMap cimport *
from MzTab cimport *
from DefaultParamHandler cimport *
from String cimport *
from ProgressLogger cimport *
from AccurateMassSearchResult cimport *
from EmpiricalFormula cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/AccurateMassSearchEngine.h>" namespace "OpenMS":

    cdef cppclass AccurateMassSearchEngine(DefaultParamHandler,ProgressLogger) :
        # wrap-inherits:
        #  DefaultParamHandler
        #  ProgressLogger
        AccurateMassSearchEngine() nogil except +
        AccurateMassSearchEngine(AccurateMassSearchEngine) nogil except + #wrap-ignore

        void queryByMZ(double observed_mz, Int observed_charge, String ion_mode,
                         libcpp_vector[ AccurateMassSearchResult ] &, EmpiricalFormula & observed_adduct) nogil except +

        void queryByFeature(Feature feature, Size feature_index, String ion_mode,
                            libcpp_vector[ AccurateMassSearchResult ] & ) nogil except +

        void queryByConsensusFeature(ConsensusFeature cfeat, Size cf_index, Size number_of_maps, String ion_mode,
                                     libcpp_vector[AccurateMassSearchResult]& results) nogil except +

        void run(FeatureMap & , MzTab & ) nogil except +
        void run(ConsensusMap&, MzTab&) nogil except +

        void init() nogil except +


## cdef extern from "<OpenMS/ANALYSIS/ID/AccurateMassSearchEngine.h>" namespace "OpenMS":
##     
##     # private default constructor
##     cdef cppclass AdductInfo "OpenMS::AdductInfo":
## 
##         AdductInfo() nogil except + #wrap-ignore
##         AdductInfo(AdductInfo) nogil except + #wrap-ignore
##         AdductInfo(String & name, EmpiricalFormula & adduct, int charge, UInt mol_multiplier) nogil except +
##         double getNeutralMass(double observed_mz) nogil except +
##         double getMZ(double neutral_mass) nogil except +
##         bool isCompatible(EmpiricalFormula db_entry) nogil except +
##         int getCharge() nogil except +
##         String getName() nogil except +
##         # AdductInfo parseAdductString(String & adduct) nogil except +
## 



from Types cimport *
from String cimport *
from Base64 cimport *

cdef extern from "<OpenMS/FORMAT/MSNumpressCoder.h>" namespace "OpenMS":

    cdef cppclass MSNumpressCoder:

        MSNumpressCoder() nogil except +
        MSNumpressCoder(MSNumpressCoder) nogil except +

        void encodeNP(libcpp_vector[double] in_,
                      String & result,
                      bool zlib_compression,
                      NumpressConfig config) nogil except +

        void decodeNP(const String& in_,
                     libcpp_vector[double] & out,
                     bool zlib_compression,
                     NumpressConfig config) nogil except +

        void encodeNPRaw(libcpp_vector[ double ] in_,
                         String & result, 
                         NumpressConfig config) nogil except +

        void decodeNPRaw(const String& in_,
                         libcpp_vector[ double ] & out,
                         NumpressConfig config) nogil except +

cdef extern from "<OpenMS/FORMAT/MSNumpressCoder.h>" namespace "OpenMS::MSNumpressCoder":

    cdef enum NumpressCompression:
      # wrap-attach:
      #     MSNumpressCoder
      NONE,
      LINEAR,
      PIC,
      SLOF,
      SIZE_OF_NUMPRESSCOMPRESSION

    cdef cppclass NumpressConfig:

      NumpressConfig() nogil except +
      NumpressConfig(NumpressConfig) nogil except +

      double numpressFixedPoint # fixed point for numpress algorithms
      double numpressErrorTolerance # check error tolerance after encoding, guarantee abs(1.0-(encoded/decoded)) <= this, 0=do not guarantee anything
      NumpressCompression np_compression # which compression schema to use
      bool estimate_fixed_point # whether to estimate the fixed point or use the one proved with numpressFixedPoint
      double linear_fp_mass_acc # desired mass accuracy for linear encoding (-1 no effect, use 0.0001 for 0.2 ppm accuracy @ 500 m/z)

      void setCompression(const String & compression) nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from DPosition cimport *

cdef extern from "<OpenMS/COMPARISON/CLUSTERING/ClusteringGrid.h>" namespace "OpenMS":
    
    cdef cppclass ClusteringGrid "OpenMS::ClusteringGrid":
        ClusteringGrid(ClusteringGrid) nogil except + #wrap-ignore
        ClusteringGrid(libcpp_vector[ double ] & grid_spacing_x, libcpp_vector[ double ] & grid_spacing_y) nogil except +
        libcpp_vector[ double ] getGridSpacingX() nogil except +
        libcpp_vector[ double ] getGridSpacingY() nogil except +
        void addCluster(libcpp_pair[int,int] cell_index, int & cluster_index) nogil except +
        void removeCluster(libcpp_pair[int,int] cell_index, int & cluster_index) nogil except +
        void removeAllClusters() nogil except +
        # NAMESPACE # std::list[ int ] getClusters(CellIndex & cell_index) nogil except +
        libcpp_pair[int,int] getIndex(DPosition2 position) nogil except +
        bool isNonEmptyCell(libcpp_pair[int,int] cell_index) nogil except +
        int getCellCount() nogil except +



from Types cimport *
from String cimport *
from DefaultParamHandler cimport *
from MultiplexDeltaMasses cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/MultiplexDeltaMassesGenerator.h>" namespace "OpenMS":
    
    cdef cppclass MultiplexDeltaMassesGenerator(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        MultiplexDeltaMassesGenerator() nogil except +
        MultiplexDeltaMassesGenerator(MultiplexDeltaMassesGenerator) nogil except + #wrap-ignore
        MultiplexDeltaMassesGenerator(String labels, int missed_cleavages, libcpp_map[ String, double ] label_mass_shift) nogil except +

        void generateKnockoutDeltaMasses() nogil except +
        libcpp_vector[ MultiplexDeltaMasses ] getDeltaMassesList() nogil except +
        # libcpp_vector[ libcpp_vector[ String ] ] getSamplesLabelsList() nogil except +
        String getLabelShort(String label) nogil except +
        String getLabelLong(String label) nogil except +

        # missing multiset support
        # NAMESPACE # MultiplexDeltaMasses::LabelSet extractLabelSet(AASequence sequence) nogil except +

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/MultiplexDeltaMassesGenerator.h>" namespace "OpenMS::MultiplexDeltaMassesGenerator":
    
    cdef cppclass MultiplexDeltaMassesGenerator_Label "OpenMS::MultiplexDeltaMassesGenerator::Label":

        MultiplexDeltaMassesGenerator_Label(MultiplexDeltaMassesGenerator_Label) nogil except + #wrap-ignore
        MultiplexDeltaMassesGenerator_Label(String sn, String ln, String d, double dm) nogil except +

        String short_name
        String long_name
        String description
        double delta_mass





from Types cimport *
from libcpp.map cimport map as libcpp_map
from libcpp.string cimport string as libcpp_string
from libcpp.vector cimport vector as libcpp_vector
from String cimport *

cdef extern from "<OpenMS/FORMAT/OSWFile.h>" namespace "OpenMS":
    
    cdef cppclass OSWFile "OpenMS::OSWFile":
        OSWFile() nogil except +
        OSWFile(OSWFile) nogil except + #wrap-ignore

        # Cannot wrap string stream
        # void read(const libcpp_string & in_osw,
        #          const libcpp_string & osw_level,
        #          libcpp_stringstream & pin_output,
        #          const double & ipf_max_peakgroup_pep,
        #          const double & ipf_max_transition_isotope_overlap,
        #          const double & ipf_min_transition_sn) nogil except +

        # NESTED STL # void write(const libcpp_string & in_osw, const libcpp_string & osw_level, const libcpp_map[ libcpp_string, libcpp_vector[ double ] ] & features) nogil except +






    # NOTE: using shared_ptr for a singleton will lead to segfaults, use raw ptr instead
    # cdef AutowrapPtrHolder[_ResidueDB] inst

    def __init__(self):
      self.inst = AutowrapPtrHolder[_ResidueDB](_getInstance_ResidueDB())

    def __dealloc__(self):
      # Careful here, the wrapped ptr is a single instance and we should not
      # reset it, therefore use 'wrap-manual-dealloc'
      pass




# Python script to compile Cython to C++
#
# this script compiles the *.pyx infile (pyopenms/pyopenms.pyx) to .cpp
#
# It can be run in the build process of pyopenms when manual changes to the pyx
# files need to be made (not recommended!).
#
# == taken from autowrap/Main.py run(), lower part
from __future__ import print_function

infile  = "pyopenms/pyopenms.pyx"

import cPickle
persisted_data_path = "include_dir.bin"
autowrap_include_dirs = cPickle.load(open(persisted_data_path, "rb"))
# autowrap_include_dirs = ['/usr/local/lib/python2.7/dist-packages/autowrap/data_files/boost', '/usr/local/lib/python2.7/dist-packages/autowrap/data_files', '/path/to/pyOpenMS/pxds', 'from Map cimport Map as _Map']


from Cython.Compiler.Main import compile as cy_compile, CompilationOptions
from Cython.Compiler.Options import directive_defaults
directive_defaults["boundscheck"] = False
directive_defaults["wraparound"] = False

options = dict(include_path=autowrap_include_dirs,
               compiler_directives=directive_defaults,
               #output_dir=".",
               #gdb_debug=True,
               cplus=True)

print("Compiling with Cython the file", infile)
print("Using include_path", autowrap_include_dirs)
options = CompilationOptions(**options)
cy_compile(infile, options=options)
print("Success!")







    def transform(self, *args):
        if (len(args)==2):
             self._transform_1(*args)
        elif (len(args)==3):
             self._transform_2(*args)
        elif (len(args)==4):
             self._transform_3(*args)
        elif (len(args)==5):
             self._transform_4(*args)
        else:
           raise Exception('can not handle type of %s' % (args,))

        # void transform(const String&, IMSDataConsumer[Peak1D, ChromatogramPeak] *) nogil except + # wrap-ignore
        # void transform(const String&, IMSDataConsumer[Peak1D, ChromatogramPeak] *, bool skip_full_count, bool skip_first_pass) nogil except +
        # void transform(const String&, IMSDataConsumer[Peak1D, ChromatogramPeak] *, MSExperiment& e) nogil except + # wrap-ignore
        # void transform(const String&, IMSDataConsumer[Peak1D, ChromatogramPeak] *, MSExperiment& e, bool skip_full_count, bool skip_first_pass) nogil except + # wrap-ignore

    def _transform_4(self, path, transformer, MSExperiment exp, bool skip_full_count, bool skip_first_pass):
        assert isinstance(exp, MSExperiment), 'arg exp wrong type'
        assert (isinstance(path, str) or isinstance(path, unicode) or isinstance(path, bytes) or isinstance(path, String)), 'arg path wrong type'

        assert hasattr(transformer, "consumeSpectrum"), "expected method consumeSpectrum"
        assert hasattr(transformer, "consumeChromatogram"), "expected method consumeChromatogram"
        assert hasattr(transformer, "setExpectedSize"), "expected method setExpectedSize"
        assert hasattr(transformer, "setExperimentalSettings"), "expected method setExperimentalSettings"
        cdef _PythonMSDataConsumer * consumer
        consumer = new _PythonMSDataConsumer(transformer,
                                             _wrap_MSSpectrum_mzml,
                                             _wrap_MSChromatogram_mzml,
                                             _wrap_ExperimentalSettings_mzml)

        try:
            self.inst.get().transform(deref((convString(path)).get()), consumer, deref(exp.inst.get()), skip_full_count, skip_first_pass)
        finally:
            del consumer

    def _transform_2(self, path, transformer, MSExperiment exp):
        assert isinstance(exp, MSExperiment), 'arg exp wrong type'
        assert (isinstance(path, str) or isinstance(path, unicode) or isinstance(path, bytes) or isinstance(path, String)), 'arg path wrong type'

        assert hasattr(transformer, "consumeSpectrum"), "expected method consumeSpectrum"
        assert hasattr(transformer, "consumeChromatogram"), "expected method consumeChromatogram"
        assert hasattr(transformer, "setExpectedSize"), "expected method setExpectedSize"
        assert hasattr(transformer, "setExperimentalSettings"), "expected method setExperimentalSettings"
        cdef _PythonMSDataConsumer * consumer
        consumer = new _PythonMSDataConsumer(transformer,
                                             _wrap_MSSpectrum_mzml,
                                             _wrap_MSChromatogram_mzml,
                                             _wrap_ExperimentalSettings_mzml)

        try:
            self.inst.get().transform(deref((convString(path)).get()), consumer, deref(exp.inst.get()) )
        finally:
            del consumer

    def _transform_3(self, path, transformer, bool skip_full_count, bool skip_first_pass):
        assert (isinstance(path, str) or isinstance(path, unicode) or isinstance(path, bytes) or isinstance(path, String)), 'arg path wrong type'

        assert hasattr(transformer, "consumeSpectrum"), "expected method consumeSpectrum"
        assert hasattr(transformer, "consumeChromatogram"), "expected method consumeChromatogram"
        assert hasattr(transformer, "setExpectedSize"), "expected method setExpectedSize"
        assert hasattr(transformer, "setExperimentalSettings"), "expected method setExperimentalSettings"
        cdef _PythonMSDataConsumer * consumer
        consumer = new _PythonMSDataConsumer(transformer,
                                             _wrap_MSSpectrum_mzml,
                                             _wrap_MSChromatogram_mzml,
                                             _wrap_ExperimentalSettings_mzml)

        try:
            self.inst.get().transform(deref((convString(path)).get()), consumer, skip_full_count, skip_first_pass)
        finally:
            del consumer


    def _transform_1(self, path, transformer):
        assert (isinstance(path, str) or isinstance(path, unicode) or isinstance(path, bytes) or isinstance(path, String)), 'arg path wrong type'

        assert hasattr(transformer, "consumeSpectrum"), "expected method consumeSpectrum"
        assert hasattr(transformer, "consumeChromatogram"), "expected method consumeChromatogram"
        assert hasattr(transformer, "setExpectedSize"), "expected method setExpectedSize"
        assert hasattr(transformer, "setExperimentalSettings"), "expected method setExperimentalSettings"
        cdef _PythonMSDataConsumer * consumer
        consumer = new _PythonMSDataConsumer(transformer,
                                             _wrap_MSSpectrum_mzml,
                                             _wrap_MSChromatogram_mzml,
                                             _wrap_ExperimentalSettings_mzml)

        try:
            self.inst.get().transform(deref((convString(path)).get()), consumer)
        finally:
            del consumer


cdef _wrap_MSSpectrum_mzml(const _MSSpectrum & _spec):
    cdef MSSpectrum spec = MSSpectrum.__new__(MSSpectrum)
    spec.inst = shared_ptr[_MSSpectrum](new _MSSpectrum(_spec))
    return spec


cdef _wrap_MSChromatogram_mzml(const _MSChromatogram & _chromo):
    cdef MSChromatogram chromo = MSChromatogram.__new__(MSChromatogram)
    chromo.inst = shared_ptr[_MSChromatogram](new _MSChromatogram(_chromo))
    return chromo


cdef _wrap_ExperimentalSettings_mzml(const _ExperimentalSettings & _exp):
    cdef ExperimentalSettings exp = ExperimentalSettings.__new__(ExperimentalSettings)
    exp.inst = shared_ptr[_ExperimentalSettings](new _ExperimentalSettings(_exp))
    return exp



from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from libcpp.pair cimport pair as libcpp_pair
from libcpp cimport bool
from MSSpectrum cimport *
from MSExperiment cimport *
from DataArrays cimport *
from SimpleTSGXLMS cimport SimplePeak


cdef extern from "<OpenMS/ANALYSIS/XLMS/OPXLSpectrumProcessingAlgorithms.h>" namespace "OpenMS":

    cdef cppclass OPXLSpectrumProcessingAlgorithms:

        OPXLSpectrumProcessingAlgorithms(OPXLSpectrumProcessingAlgorithms) nogil except +
        OPXLSpectrumProcessingAlgorithms() nogil except +

        MSSpectrum mergeAnnotatedSpectra(MSSpectrum& first_spectrum,
                                         MSSpectrum& second_spectrum) nogil except +

        MSExperiment preprocessSpectra(MSExperiment& exp,
                                        double fragment_mass_tolerance,
                                        bool fragment_mass_tolerance_unit_ppm,
                                        Size peptide_min_size,
                                        Int min_precursor_charge,
                                        Int max_precursor_charge,
                                        bool deisotope,
                                        bool labeled) nogil except +

        void getSpectrumAlignmentFastCharge(libcpp_vector[ libcpp_pair[ size_t, size_t ] ]& alignment,
                                            double fragment_mass_tolerance,
                                            bool fragment_mass_tolerance_unit_ppm,
                                            const MSSpectrum& theo_spectrum,
                                            const MSSpectrum& exp_spectrum,
                                            const IntegerDataArray& theo_charges,
                                            const IntegerDataArray& exp_charges,
                                            FloatDataArray& ppm_error_array,
                                            double intensity_cutoff) nogil except +

        void getSpectrumAlignmentSimple(libcpp_vector[ libcpp_pair[ size_t, size_t ] ]& alignment,
                                            double fragment_mass_tolerance,
                                            bool fragment_mass_tolerance_unit_ppm,
                                            const libcpp_vector[ SimplePeak ]& theo_spectrum,
                                            const MSSpectrum& exp_spectrum,
                                            const IntegerDataArray& exp_charges) nogil except +



from libcpp cimport bool
from MetaInfoInterface cimport *
from libcpp.vector cimport vector as libcpp_vector
from DataValue cimport *
from String cimport *

cdef extern from "<OpenMS/METADATA/Acquisition.h>" namespace "OpenMS":
	
	cdef cppclass Acquisition(MetaInfoInterface):
		# wrap-inherits:
		#  MetaInfoInterface

		Acquisition() nogil except +
		Acquisition(Acquisition) nogil except +

		bool operator==(Acquisition &rhs) nogil except +
		bool operator!=(Acquisition &rhs) nogil except +
		String getIdentifier() nogil except +
		void setIdentifier(const String &identifier) nogil except +



from libcpp cimport bool
from String cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/CVMappingTerm.h>" namespace "OpenMS":

    cdef cppclass CVMappingTerm:

        CVMappingTerm()               nogil except +
        CVMappingTerm(CVMappingTerm)  nogil except +

        # sets the accession string of the term
        void setAccession(String accession) nogil except +

        # returns the accession string of the term
        String getAccession() nogil except +

        # sets whether the term name should be used, instead of the accession
        void setUseTermName(bool use_term_name) nogil except +

        # returns whether the term name should be used, instead of the accession
        bool getUseTermName() nogil except +

        # sets whether the term itself can be used (or only its children)
        void setUseTerm(bool use_term) nogil except +

        # returns true if the term can be used, false if only children are allowed
        bool getUseTerm() nogil except +

        # sets the name of the term
        void setTermName(String term_name) nogil except +

        # returns the name of the term
        String getTermName() nogil except +

        # sets whether this term can be repeated
        void setIsRepeatable(bool is_repeatable) nogil except +

        # returns true if this term can be repeated, false otherwise
        bool getIsRepeatable() nogil except +

        # sets whether children of this term are allowed
        void setAllowChildren(bool allow_children) nogil except +

        # returns true if the children of this term are allowed to be used
        bool getAllowChildren() nogil except +

        # sets the cv identifier reference string, e.g. UO for unit obo
        void setCVIdentifierRef(String cv_identifier_ref) nogil except +

        # returns the cv identifier reference string
        String getCVIdentifierRef() nogil except +

        # equality operator
        bool operator==(CVMappingTerm rhs) nogil except +

        # inequality operator
        bool operator!=(CVMappingTerm rhs) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp.map cimport map as libcpp_map
from libcpp.pair cimport pair as libcpp_pair
from libcpp cimport bool

from ConsensusMap cimport *
from DefaultParamHandler cimport *
from Feature cimport *
from FeatureMap cimport *
from String cimport *

from ProteinIdentification cimport *
from PeptideIdentification cimport *

from MSExperiment cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/IDRipper.h>" namespace "OpenMS":

    cdef cppclass IDRipper(DefaultParamHandler):
        # wrap-inherits:
        #    DefaultParamHandler

        IDRipper() nogil except +
        IDRipper(IDRipper) nogil except +   # wrap-ignore

        # see additional pyx file in ./addons
        void rip( 
          libcpp_map[String, libcpp_pair[ libcpp_vector[ProteinIdentification],
          libcpp_vector[PeptideIdentification]]] & ripped,
          libcpp_vector[ProteinIdentification] & proteins,
          libcpp_vector[PeptideIdentification] & peptides) # wrap-ignore




from libcpp cimport bool
from String cimport *
from IsotopeWavelet cimport *
from FeatureMap cimport *
from MSExperiment cimport *
from MSSpectrum cimport *
from Peak1D cimport *
# from LinearRegression cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/IsotopeWaveletTransform.h>" namespace "OpenMS":

    cdef cppclass IsotopeWaveletTransform[PeakT]:
        # wrap-instances:
        #   IsotopeWaveletTransform := IsotopeWaveletTransform[Peak1D]

        # IsotopeWaveletTransform() nogil except +
        IsotopeWaveletTransform(IsotopeWaveletTransform) nogil except + #wrap-ignore
        IsotopeWaveletTransform(double min_mz, double max_mz, UInt max_charge, Size max_scan_size, bool hr_data, String intenstype) nogil except +

        void getTransform(MSSpectrum &c_trans, MSSpectrum &c_ref, UInt c) nogil except +
        void getTransformHighRes(MSSpectrum &c_trans, MSSpectrum &c_ref, UInt c) nogil except +
        void identifyCharge(MSSpectrum &candidates, MSSpectrum &ref, UInt scan_index, UInt c, double ampl_cutoff, bool check_PPMs) nogil except +
        void initializeScan(MSSpectrum &c_ref, UInt c) nogil except +
        void updateBoxStates(MSExperiment &map_, Size scan_index, UInt RT_interleave, UInt RT_votes_cutoff, Int front_bound, Int end_bound) nogil except +
        # void mergeFeatures(IsotopeWaveletTransform[ PeakT ] *later_iwt, UInt RT_interleave, UInt RT_votes_cutoff) nogil except +
        FeatureMap mapSeeds2Features(MSExperiment &map_, UInt RT_votes_cutoff) nogil except +
        ## std::multimap[ double, Box ] getClosedBoxes() nogil except +
        ## double getLinearInterpolation(typename MSSpectrum::const_iterator &left_iter, double mz_pos, typename MSSpectrum::const_iterator &right_iter) nogil except +
        double getLinearInterpolation(double mz_a, double intens_a, double mz_pos, double mz_b, double intens_b) nogil except +
        double getSigma() nogil except +
        void setSigma(double sigma) nogil except +
        void computeMinSpacing(MSSpectrum &c_ref) nogil except +
        double getMinSpacing() nogil except +
        Size getMaxScanSize() nogil except +

# TODO C++ compiler errors
# pyopenms/pyopenms.cpp: error: template argument 1 is invalid
# on boost::shared_ptr<OpenMS::IsotopeWaveletTransform::TransSpectrum> inst;
# cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/IsotopeWaveletTransform.h>" namespace "OpenMS::IsotopeWaveletTransform":

#     cdef cppclass TransSpectrum "OpenMS::IsotopeWaveletTransform::TransSpectrum":
#         TransSpectrum() nogil except +
#         TransSpectrum(TransSpectrum) nogil except + #wrap-ignore

#         # POINTER #  TransSpectrum(MSSpectrum * reference) nogil except +
#         void destroy() nogil except +
#         double getRT() nogil except +
#         double getMZ(UInt i) nogil except +
#         double getRefIntensity(UInt i) nogil except +
#         double getTransIntensity(UInt i) nogil except +
#         void setTransIntensity(UInt i, double intens) nogil except +
#         Size size() nogil except +
#         # POINTER # MSSpectrum * getRefSpectrum() nogil except +
#         # POINTER # MSSpectrum * getRefSpectrum() nogil except +
#         # NAMESPACE # MSSpectrum::const_iterator MZBegin(double mz) nogil except +
#         # NAMESPACE # MSSpectrum::const_iterator MZEnd(double mz) nogil except +
#         # NAMESPACE # MSSpectrum::const_iterator end() nogil except +
#         # NAMESPACE # MSSpectrum::const_iterator begin() nogil except +
#

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/IsotopeWaveletTransform.h>" namespace "OpenMS::IsotopeWaveletTransform":

    cdef cppclass BoxElement "OpenMS::IsotopeWaveletTransform::BoxElement":
        BoxElement(BoxElement) nogil except + #wrap-ignore
        double mz
        UInt c
        double score
        double intens
        double ref_intens
        double RT
        UInt RT_index
        UInt MZ_begin
        UInt MZ_end




from Types cimport *

from RNPxlMarkerIonExtractor cimport *
# from ListUtilsIO cimport *
from PeptideIdentification cimport *
from MSSpectrum cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/ANALYSIS/RNPXL/RNPxlReport.h>" namespace "OpenMS":
    
    cdef cppclass RNPxlReport "OpenMS::RNPxlReport":
        RNPxlReport(RNPxlReport) nogil except + #wrap-ignore
        libcpp_vector[ RNPxlReportRow ] annotate(MSExperiment & spectra,
                                                 libcpp_vector[ PeptideIdentification ] & peptide_ids,
                                                 double marker_ions_tolerance) nogil except +


cdef extern from "<OpenMS/ANALYSIS/RNPXL/RNPxlReport.h>" namespace "OpenMS":
    
    cdef cppclass RNPxlReportRowHeader "OpenMS::RNPxlReportRowHeader":
        RNPxlReportRowHeader(RNPxlReportRowHeader) nogil except + #wrap-ignore
        String getString(const String & separator) nogil except +

cdef extern from "<OpenMS/ANALYSIS/RNPXL/RNPxlReport.h>" namespace "OpenMS":
    
    cdef cppclass RNPxlReportRow "OpenMS::RNPxlReportRow":
        RNPxlReportRow(RNPxlReportRow) nogil except + #wrap-ignore

        bool no_id
        double rt
        double original_mz
        String accessions
        String RNA
        String peptide
        double best_localization_score
        String localization_scores
        String best_localization
        Int charge
        double score
        double peptide_weight
        double RNA_weight
        double xl_weight
        double abs_prec_error
        double rel_prec_error
        libcpp_map[String, libcpp_vector[ libcpp_pair[double, double] ] ] marker_ions # RNPxlMarkerIonExtractor::MarkerIonsType marker_ions
        double m_H
        double m_2H
        double m_3H
        double m_4H
        int rank

        String getString(const String & separator) nogil except +




from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from ProgressLogger cimport *
from TargetedExperiment cimport *
from LightTargetedExperiment cimport *
from TransformationDescription cimport *
from libcpp cimport bool

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/OpenSwathHelper.h>" namespace "OpenMS":

    cdef cppclass OpenSwathHelper:

        bool checkSwathMapAndSelectTransitions(
                                MSExperiment & exp, 
                                TargetedExperiment & targeted_exp,
                                TargetedExperiment & transition_exp_used,
                                double min_upper_edge_dist
                                ) nogil except +


        libcpp_pair[double, double] estimateRTRange(LightTargetedExperiment exp) nogil except +

        String computePrecursorId(const String & transition_group_id, int isotope) nogil except +

        # static std::map<std::string, double> simpleFindBestFeature(
        #     OpenMS::MRMFeatureFinderScoring::TransitionGroupMapType & transition_group_map, 
        #     bool useQualCutoff = false, double qualCutoff = 0.0);





from Types cimport *
from DataValue cimport *
from String cimport *
from MetaInfoRegistry cimport *

cdef extern from "<OpenMS/METADATA/MetaInfoInterface.h>" namespace "OpenMS":

    cdef cppclass MetaInfoInterface:
        # wrap-doc:
        #   Interface for classes that can store arbitrary meta information
        #   (Type-Name-Value tuples).
        #   -----
        #   MetaInfoInterface is a base class for all classes that use one MetaInfo
        #   object as member.  If you want to add meta information to a class, let it
        #   publicly inherit the MetaInfoInterface.  Meta information is an array of
        #   Type-Name-Value tuples.
        #   -----
        #   Usage:
        #     k = []
        #     exp.getKeys(k) # explore available key-value pairs
        #     exp.getMetaValue("someMetaName")
        #   -----

        MetaInfoInterface() nogil except +
        MetaInfoInterface(MetaInfoInterface) nogil except +

        bool operator==(MetaInfoInterface) nogil except +
        bool operator!=(MetaInfoInterface) nogil except +
        bool isMetaEmpty() nogil except +
        void clearMetaInfo() nogil except +

        MetaInfoRegistry metaRegistry() nogil except +

        # Cython has a problem with inheritance of overloaded methods, so we
        # can only declare one of the methods here (most people will want
        # to use the String-based methods).
        #
        void getKeys(libcpp_vector[String] & keys)
        #void getKeys(libcpp_vector[unsigned int] & keys)
        DataValue getMetaValue(String) nogil except +
        #DataValue getMetaValue(String, DataValue) nogil except +
        #DataValue getMetaValue(unsigned int) nogil except +
        #DataValue getMetaValue(unsigned int, DataValue) nogil except +
        void setMetaValue(String, DataValue) nogil except +
        #void setMetaValue(unsigned int, DataValue) nogil except +
        bool metaValueExists(String) nogil except +
        #bool metaValueExists(unsigned int) nogil except +
        void removeMetaValue(String) nogil except +
        #void removeMetaValue(unsigned int) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp cimport bool
from Types cimport *
from Map cimport *
from Peak2D cimport *
from String cimport *
from StringList cimport *
from Matrix cimport *

# ctypedef Map<Int, ChannelInfo> ChannelMapType; 
# ctypedef std::vector<Matrix<double> > IsotopeMatrices;
cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ItraqConstants.h>" namespace "OpenMS":
    
    cdef cppclass ItraqConstants "OpenMS::ItraqConstants":
        ItraqConstants() nogil except + 
        ItraqConstants(ItraqConstants) nogil except + #wrap-ignore

        # Int CHANNEL_COUNT()
        # Int CHANNELS_FOURPLEX()
        # Int CHANNELS_EIGHTPLEX()
        # Int CHANNELS_TMT_SIXPLEX()
        # double ISOTOPECORRECTIONS_FOURPLEX()
        # double ISOTOPECORRECTIONS_EIGHTPLEX()
        # double ISOTOPECORRECTIONS_TMT_SIXPLEX()

        StringList getIsotopeMatrixAsStringList(int itraq_type, libcpp_vector[Matrix[double] ] & isotope_corrections) nogil except +
        void updateIsotopeMatrixFromStringList(int itraq_type, StringList & channels, libcpp_vector[Matrix[double] ] & isotope_corrections) nogil except +
        # void initChannelMap(int itraq_type, ChannelMapType & map_) nogil except +
        # void updateChannelMap(StringList & active_channels, ChannelMapType & map_) nogil except +
        Matrix[ double ] translateIsotopeMatrix(int & itraq_type, libcpp_vector[Matrix[double] ] & isotope_corrections) nogil except +

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ItraqConstants.h>" namespace "OpenMS::ItraqConstants":

    cdef enum ITRAQ_TYPES:

        FOURPLEX, EIGHTPLEX, TMT_SIXPLEX, SIZE_OF_ITRAQ_TYPES

    cdef cppclass ChannelInfo "OpenMS::ItraqConstants::ChannelInfo":
        ChannelInfo(ChannelInfo) nogil except + #wrap-ignore
        libcpp_string description
        Int name
        Int id
        double center
        bool active




from Types cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from MetaInfoInterface cimport *
from CVTerm cimport *
from Map cimport *

cdef extern from "<OpenMS/METADATA/CVTermListInterface.h>" namespace "OpenMS":
    
    cdef cppclass CVTermListInterface(MetaInfoInterface) :
        # wrap-inherits:
        #  MetaInfoInterface

        CVTermListInterface() nogil except +
        CVTermListInterface(CVTermListInterface) nogil except +

        bool operator==(CVTermListInterface & rhs) nogil except +
        bool operator!=(CVTermListInterface & rhs) nogil except +

        void replaceCVTerms(Map[ String, libcpp_vector[ CVTerm ] ] & cv_terms) nogil except +
        void setCVTerms(libcpp_vector[ CVTerm ] & terms) nogil except +

        ######################################################################
        # Cython has a problem with inheritance of overloaded methods, so we
        # can only declare one of the two methods here.
        void replaceCVTerm(CVTerm & cv_term) nogil except +
        void replaceCVTerms(libcpp_vector[ CVTerm ] & cv_terms, const String & accession) nogil except +
        # void replaceCVTerms(Map[ String, libcpp_vector[ CVTerm ] ] & cv_term_map) nogil except +

        void consumeCVTerms(Map[ String, libcpp_vector[ CVTerm ] ] & cv_term_map) nogil except +
        Map[ String, libcpp_vector[ CVTerm ] ]  getCVTerms() nogil except +


        void addCVTerm(CVTerm & term) nogil except +
        bool hasCVTerm(const String & accession) nogil except +
        bool empty() nogil except +




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from Residue cimport *
from DefaultParamHandler cimport *
from AASequence cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/ProtonDistributionModel.h>" namespace "OpenMS":
    
    cdef cppclass ProtonDistributionModel(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        ProtonDistributionModel() nogil except +
        ProtonDistributionModel(ProtonDistributionModel) nogil except +
        void getProtonDistribution(libcpp_vector[ double ] & bb_charges, libcpp_vector[ double ] & sc_charges, AASequence & peptide, Int charge, ResidueType res_type) nogil except +
        void getChargeStateIntensities(AASequence & peptide, AASequence & n_term_ion, AASequence & c_term_ion, Int charge, ResidueType n_term_type, libcpp_vector[ double ] & n_term_intensities, libcpp_vector[ double ] & c_term_intensities, FragmentationType type_) nogil except +
        void setPeptideProtonDistribution(libcpp_vector[ double ] & bb_charge, libcpp_vector[ double ] & sc_charge) nogil except +

cdef extern from "<OpenMS/ANALYSIS/ID/ProtonDistributionModel.h>" namespace "OpenMS::ProtonDistributionModel":
    cdef enum FragmentationType "OpenMS::ProtonDistributionModel::FragmentationType":
        #wrap-attach:
        #    ProtonDistributionModel
        ChargeDirected
        ChargeRemote
        SideChain




from Types cimport *
from IsobaricQuantitationMethod cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/TMTSixteenPlexQuantitationMethod.h>" namespace "OpenMS":
    
    cdef cppclass TMTSixteenPlexQuantitationMethod(IsobaricQuantitationMethod) :
        # wrap-inherits:
        #  IsobaricQuantitationMethod
        TMTSixteenPlexQuantitationMethod() nogil except +
        TMTSixteenPlexQuantitationMethod(TMTSixteenPlexQuantitationMethod) nogil except +



import unittest
import os

import pyopenms
class TestAcquisitionInfo(unittest.TestCase):

	def setUp(self):
		dirname = os.path.dirname(os.path.abspath(__file__))
		self.filename_mzml = os.path.join(dirname, "test.mzML").encode()

	def test_acquisitioninfomemberaccess(self):
		exp = pyopenms.MSExperiment()
		pyopenms.MzMLFile().load(self.filename_mzml, exp)

		# Basically test that the output is non-zero (e.g. the data is
		# correctly relayed to python)
		# The functionality is not tested here!

		# starting point
		self.assertEqual(exp[0].getAcquisitionInfo().size(), 1)
		self.assertNotEqual(exp[0].getAcquisitionInfo().size(), 0)

		# metainfo
		exp[0].getAcquisitionInfo().size()  # is 1
		self.assertEqual(exp[0].getAcquisitionInfo()[0].isMetaEmpty(), True)  # is True

                spectra = exp.getSpectra()
                aqis = spectra[0].getAcquisitionInfo()
                aqi = aqis[0] # get a copy
                aqi.setMetaValue('key', 420) # modify it
                aqis[0] = aqi # and set entry
                spectra[0].setAcquisitionInfo(aqis)
                exp.setSpectra(spectra)
                self.assertEqual(exp[0].getAcquisitionInfo()[0].getMetaValue('key'), 420)  # should be 420

		acin = pyopenms.Acquisition()
		acin.setMetaValue('key', 42)
		self.assertEqual(acin.getMetaValue('key'), 42)  # is 42
		self.assertEqual(acin.isMetaEmpty(), False)  # is False

		# list/vector assignment
		magicnumber = 3
		neac = pyopenms.AcquisitionInfo()
		for i in range(0,magicnumber):
			neac.push_back(acin)

		self.assertEqual(neac.size(), magicnumber)  # is magicnumber

		# iteration
		for i in neac:
			self.assertEqual(i.isMetaEmpty(), False)  # always is False
		
		# accession already tested in 2nd section
		tmp = exp.getSpectra()
		tmp[0].setAcquisitionInfo(neac)
		exp.setSpectra(tmp)

		self.assertEqual(exp[0].getAcquisitionInfo().size(), magicnumber)  # should be magicnumber

		for i in exp[0].getAcquisitionInfo():
			self.assertEqual(i.isMetaEmpty(), False)  # should always be False

		# resize
		neac.resize(0)
		self.assertEqual(neac.size(), 0)

if __name__ == '__main__':
	unittest.main()



from libcpp cimport bool
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from Adduct cimport *
from StringList cimport *
from Map cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/Compomer.h>" namespace "OpenMS":

    cdef cppclass Compomer:
  
        Compomer() nogil except +
        Compomer(Compomer) nogil except + 
  
        void add(Adduct & a, UInt side) nogil except +

        bool isConflicting(Compomer & cmp, UInt side_this, UInt side_other) nogil except +

        # /// set an Id which allows unique identification of a compomer
        void setID(Size id) nogil except +

        # /// return Id which allows unique identification of this compomer
        Size getID() nogil except +

        # /// left and right adducts of this compomer
        # TODO OpenMS Map type
        libcpp_vector[Map[String, Adduct] ] getComponent() nogil except + # wrap-ignore
    
        # /// net charge of compomer (i.e. difference between left and right side of compomer)
        Int getNetCharge() nogil except +

        # /// mass of all contained adducts
        double getMass() nogil except +

        # /// summed positive charges of contained adducts
        Int getPositiveCharges() nogil except +

        # /// summed negative charges of contained adducts
        Int getNegativeCharges() nogil except +

        # /// return log probability
        double getLogP() nogil except +

        # /// return log probability
        double getRTShift() nogil except +

        # /// get adducts with their abundance as compact string for both sides
        String getAdductsAsString() nogil except +

        # /// get adducts with their abundance as compact string (amounts are absolute unless side=BOTH)
        # /// @param side Use LEFT for left, RIGHT for right
        String getAdductsAsString(UInt side) nogil except +

        # /// check if Compomer only contains a single adduct on side @p side
        bool isSingleAdduct(Adduct & a, UInt side) nogil except +

        Compomer removeAdduct(Adduct & a) nogil except +

        Compomer removeAdduct(Adduct & a, UInt side) nogil except +

        StringList getLabels(UInt side) nogil except +

        # void add(CompomerSide & add_side, UInt side) nogil except +

cdef extern from "<OpenMS/DATASTRUCTURES/Compomer.h>" namespace "OpenMS::Compomer":
    # side of compomer (LEFT ^ substract; RIGHT ^ add)
    cdef enum SIDE: LEFT, RIGHT, BOTH
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from Adduct cimport *
from StringList cimport *
from Map cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/Compomer.h>" namespace "OpenMS":

    cdef cppclass Compomer:
  
        Compomer() nogil except +
        Compomer(Compomer) nogil except + 
  
        void add(Adduct & a, UInt side) nogil except +

        bool isConflicting(Compomer & cmp, UInt side_this, UInt side_other) nogil except +

        # /// set an Id which allows unique identification of a compomer
        void setID(Size id) nogil except +

        # /// return Id which allows unique identification of this compomer
        Size getID() nogil except +

        # /// left and right adducts of this compomer
        # TODO OpenMS Map type
        libcpp_vector[Map[String, Adduct] ] getComponent() nogil except + # wrap-ignore
    
        # /// net charge of compomer (i.e. difference between left and right side of compomer)
        Int getNetCharge() nogil except +

        # /// mass of all contained adducts
        double getMass() nogil except +

        # /// summed positive charges of contained adducts
        Int getPositiveCharges() nogil except +

        # /// summed negative charges of contained adducts
        Int getNegativeCharges() nogil except +

        # /// return log probability
        double getLogP() nogil except +

        # /// return log probability
        double getRTShift() nogil except +

        # /// get adducts with their abundance as compact string for both sides
        String getAdductsAsString() nogil except +

        # /// get adducts with their abundance as compact string (amounts are absolute unless side=BOTH)
        # /// @param side Use LEFT for left, RIGHT for right
        String getAdductsAsString(UInt side) nogil except +

        # /// check if Compomer only contains a single adduct on side @p side
        bool isSingleAdduct(Adduct & a, UInt side) nogil except +

        Compomer removeAdduct(Adduct & a) nogil except +

        Compomer removeAdduct(Adduct & a, UInt side) nogil except +

        StringList getLabels(UInt side) nogil except +

        # void add(CompomerSide & add_side, UInt side) nogil except +

cdef extern from "<OpenMS/DATASTRUCTURES/Compomer.h>" namespace "OpenMS::Compomer":
    # side of compomer (LEFT ^ substract; RIGHT ^ add)
    cdef enum SIDE: LEFT, RIGHT, BOTH



from libcpp.vector cimport vector as libcpp_vector
from libcpp.map cimport map as libcpp_map
from libcpp.pair cimport pair as libcpp_pair
from ConsensusMap cimport *
from String cimport *
from Types cimport *
from StringList cimport *
from ExperimentalSettings cimport *
from DataProcessing cimport *
from MetaInfo cimport *
from ConsensusMap cimport *
from CVTermList cimport *
from FeatureMap cimport *
from Feature cimport *
from MSExperiment cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/METADATA/MSQuantifications.h>" namespace "OpenMS":

    cdef cppclass MSQuantifications(ExperimentalSettings):
        # wrap-inherits:
        #  ExperimentalSettings
        MSQuantifications()  nogil
        MSQuantifications(MSQuantifications) nogil

        # Detailed constructor from a FeatureMap
        MSQuantifications(FeatureMap fm,
                          ExperimentalSettings& es,
                          libcpp_vector[DataProcessing]& dps) nogil except +
        # std::vector<std::vector<std::pair<String, double> > > labels = (std::vector<std::vector<std::pair<String, double> > >()));
        
        bool operator==(MSQuantifications &) nogil
        bool operator!=(MSQuantifications &) nogil

        # TODO - not implemented in OpenMS, remove from API
        # void load(String filename, bool trim_lines, Int first_n) nogil except +

        libcpp_vector[DataProcessing] getDataProcessingList() nogil except +
        libcpp_vector[Assay] getAssays() nogil except +

        # TODO - not implemented in OpenMS, remove from API
        # libcpp_map[String, Ratio] getRatios() nogil except + # wrap-ignore

        libcpp_vector[ConsensusMap] getConsensusMaps() nogil except +
        void setConsensusMaps(libcpp_vector[ConsensusMap]) nogil except +
        libcpp_vector[FeatureMap ] getFeatureMaps() nogil except +
        AnalysisSummary getAnalysisSummary() nogil except +
        void setDataProcessingList(libcpp_vector[DataProcessing] dpl) nogil except +
        void setAnalysisSummaryQuantType(QUANT_TYPES r) nogil except +
        void addConsensusMap(ConsensusMap m) nogil except +
        void assignUIDs() nogil except +
        void registerExperiment(MSExperiment exp, 
                                libcpp_vector[ libcpp_vector[ libcpp_pair[
                                  String, double] ] ] labels) nogil except + # wrap-ignore

cdef extern from "<OpenMS/METADATA/MSQuantifications.h>" namespace "OpenMS::MSQuantifications":
    # derived from processing applied
    cdef enum QUANT_TYPES:
        # wrap-attach:
        #     MSQuantifications
        MS1LABEL = 0,
        MS2LABEL,
        LABELFREE,
        SIZE_OF_QUANT_TYPES

    cdef cppclass AnalysisSummary:
        AnalysisSummary()  nogil except +
        AnalysisSummary(AnalysisSummary) nogil except +

        MetaInfo user_params_
        CVTermList cv_params_
        QUANT_TYPES quant_type_

    cdef cppclass Assay:
        Assay() nogil except +
        Assay(Assay) nogil except +

        String uid_ 
        libcpp_vector[libcpp_pair[String, double] ] mods_ # wrap-ignore
        libcpp_vector[ExperimentalSettings] raw_files_
        libcpp_map[size_t, FeatureMap ] feature_maps_ 
        # iTRAQ needs no FeatureMaps so ExperimentalSettings are not directly mapped to FeatureMaps




from Types cimport *
from libcpp cimport bool
from libcpp.map cimport map as libcpp_map
from libcpp.set cimport set as libcpp_set
# from GridFeature cimport *
from AASequence cimport *

# typedef OpenMSBoost::unordered_map<Size, std::multimap<double, GridFeature*> > NeighborMap;

cdef extern from "<OpenMS/DATASTRUCTURES/QTCluster.h>" namespace "OpenMS":
    
    cdef cppclass QTCluster "OpenMS::QTCluster":
        QTCluster(QTCluster) nogil except + #wrap-ignore

        # POINTER #  QTCluster(GridFeature * center_point, Size num_maps, double max_distance, bool use_IDs) nogil except +
        double getCenterRT() nogil except +
        double getCenterMZ() nogil except +
        Int getXCoord() nogil except +
        Int getYCoord() nogil except +
        Size size() nogil except +
        bool operator<(QTCluster & cluster) nogil except +
        # POINTER # void add(GridFeature * element, double distance) nogil except +
        # POINTER # void getElements(libcpp_map[ Size, GridFeature * ] & elements) nogil except +
        # POINTER # bool update(libcpp_map[ Size, GridFeature * ] & removed) nogil except +
        double getQuality() nogil except +
        libcpp_set[ AASequence ]  getAnnotations() nogil except +
        void setInvalid() nogil except +
        bool isInvalid() nogil except +
        void initializeCluster() nogil except +
        void finalizeCluster() nogil except +
        # NAMESPACE # # POINTER # OpenMSBoost::unordered_map[ Size, libcpp_vector[ GridFeature * ] ] getAllNeighbors() nogil except +
        # NeighborMap getNeighbors() nogil except +




from Types cimport *
from libcpp cimport bool

cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/IMSIsotopeDistribution.h>" namespace "OpenMS::ims::IMSIsotopeDistribution":

    ctypedef signed int size_type
    ctypedef double mass_type
    ctypedef double abundance_type
    ctypedef unsigned int nominal_mass_type
    ctypedef libcpp_vector[mass_type] masses_container
    ctypedef IMSIsotopeDistribution_Peak peak_type
    ctypedef libcpp_vector[peak_type] peaks_container
    ctypedef libcpp_vector[abundance_type] abundances_container
    
cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/IMSIsotopeDistribution.h>" namespace "OpenMS::ims":

    cdef cppclass IMSIsotopeDistribution "OpenMS::ims::IMSIsotopeDistribution":
        IMSIsotopeDistribution() nogil except +
        IMSIsotopeDistribution(IMSIsotopeDistribution) nogil except +
        abundance_type ABUNDANCES_SUM_ERROR
        size_type SIZE
        IMSIsotopeDistribution(nominal_mass_type nominalMass) nogil except +
        IMSIsotopeDistribution(mass_type mass) nogil except +
        IMSIsotopeDistribution(libcpp_vector[IMSIsotopeDistribution_Peak] & peaks, nominal_mass_type nominalMass) nogil except +
        size_type size() nogil except +
        bool operator==(IMSIsotopeDistribution & distribution) nogil except +
        bool operator!=(IMSIsotopeDistribution & distribution) nogil except +
        # IMSIsotopeDistribution operator*=(IMSIsotopeDistribution & distribution) nogil except +
        # IMSIsotopeDistribution operator*=(unsigned int pow_) nogil except +
        mass_type getMass(size_type i) nogil except +
        abundance_type getAbundance(size_type i) nogil except +
        mass_type getAverageMass() nogil except +
        nominal_mass_type getNominalMass() nogil except +
        void setNominalMass(nominal_mass_type nominalMass) nogil except +
        masses_container getMasses() nogil except + 
        libcpp_vector[abundance_type] getAbundances() nogil except +
        void normalize() nogil except +
        bool empty() nogil except +

cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/IMSIsotopeDistribution.h>" namespace "OpenMS::ims::IMSIsotopeDistribution":
    
    cdef cppclass IMSIsotopeDistribution_Peak "OpenMS::ims::IMSIsotopeDistribution::Peak":
        IMSIsotopeDistribution_Peak(IMSIsotopeDistribution_Peak) nogil except + #wrap-ignore
        mass_type mass
        abundance_type abundance
        IMSIsotopeDistribution_Peak(mass_type mass, abundance_type abundance) nogil except +
        bool operator==(IMSIsotopeDistribution_Peak & peak) nogil except +

